/*
 * @eventFileDataStruct.java Version 1.1 02/18/2015
 *
 * Copyright (c) 2015 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package event;

/** Class eventFileDataStruct
 *  <p> This Class holds the LAS File Well Data.
 *
 *  @version 1.1 02/18/2015
 *  @author  John R. Victorine
 */

public class eventFileDataStruct
{
  public double dNull   = -999.95; // Default Null Value of Data
  public double dStart  = 0.0;     // Starting Depth (ft)
  public double dEnd    = 0.0;     // Ending Depth (ft)

  public double dElev   = 0.0;     // Elevation

  // Curves Available

  public int    iTOP   = -1;   // Top Depth
  public int    iBASE  = -1;   // Base Depth

  public int    iDTSP  = -1;   // Time Difference Shear and Compression
  public int    iDTSPf = -1;   // Time Difference Shear fast and Compression
  public int    iDTSPs = -1;   // Time Difference Shear slow and Compression

  public int    iGR    = -1;   // Gamma Ray Curve Number in LAS File
  public int    iCGR   = -1;   // Gamma Ray Minus Uranium Curve Number in LAS File

  public int    iNPHI  = -1;   // Neutron Porosity Curve Number in LAS File
  public int    iPE    = -1;   // Photoelectric Curve Number in LAS File
  public int    iRHOB  = -1;   // Bulk Density Curve Number in LAS File
  public int    iDPHI  = -1;   // Density Porosity Curve Number in LAS File

  public int    iDTC   = -1;   // Sonic Acoustic Transit Time Curve Number in LAS File
  public int    iDTS   = -1;   // Sonic Shear Curve Number in LAS File
  public int    iDTSF  = -1;   // Sonic Fast Shear Curve Number in LAS File
  public int    iDTSS  = -1;   // Sonic Slow Shear Curve Number in LAS File
  public int    iSPHI  = -1;   // Sonic Porisity Curve Number in LAS File

  public int    iTh    = -1;   // Thorium Curve Number in LAS File
  public int    iU     = -1;   // Uranium Curve Number in LAS File
  public int    iK     = -1;   // Potasium Curve Number in LAS File

  // Computed Curves

  public int    iUmaa    = -1;  // Apparent Volumetric Photoelectric factor
  public int    iRhomaa  = -1;  // Apparent Matrix Grain Density
  public int    iDTmaa   = -1;  // Apparent Matrix Grain Acoustic transit time

    // Alternate Lithology Curves

  public int    iNeutron = -1; // Neutron Porosity
  public int    iDelta   = -1; // Neutron - Density Porosity

  // Spectral Gamma Ray Lithology Curves

  public int    iThU     = -1; // Th/U
  public int    iThK     = -1; // Th/K

  // Primary Lithology

  public int    iLITH    = -1; // Lithology determined

  // Sonic Computed Variables

  public int    iVc      = -1; // Compression Velocity (ft/sec)
  public int    iVs      = -1; // Shear Velocity (ft/sec)
  public int    iVsf     = -1; // Fast Shear Velocity (ft/sec)
  public int    iVss     = -1; // Slow Shear Velocity (ft/sec)

  public int    iTc      = -1; // Compression Time (milli-sec)
  public int    iTs      = -1; // Shear Time (milli-sec)
  public int    iTsf     = -1; // Fast Shear Time (milli-sec)
  public int    iTss     = -1; // Slow Shear Time (milli-sec)

  public int    iAIc     = -1; // Compression Acoustic Impedence (ft/sec * gm/cc)
  public int    iAIs     = -1; // Shear Acoustic Impedence (ft/sec * gm/cc)
  public int    iAIsf    = -1; // Fast Shear Acoustic Impedence (ft/sec * gm/cc)
  public int    iAIss    = -1; // Slow Shear Acoustic Impedence (ft/sec * gm/cc)

  public int    iVAc     = -1; // Average Compression Velocity (ft/sec)
  public int    iVAs     = -1; // Average Shear Velocity (ft/sec)

//  public int    iTSP     = -1; // S-P Time Difference (milli-sec)
//  public int    iTSPf    = -1; // Sf-P Time Difference (milli-sec)
//  public int    iTSPs    = -1; // Ss-P Time Difference (milli-sec)

  // Lithofacies Log Curves

  public int    iRows      = 0;    // Number of Rows

  public String sName[]    = null; // Name of bed

  public double depths[]   = null; // Array of Depth Data Values
  public double dTime[]    = null; // Array of Time  Data Values

  public double dTop[]     = null; // Array of Depth Start Data Values
  public double dBase[]    = null; // Array of Depth End Data Values

  public double dDTSP[]    = null; // Array of Time Difference S and P
  public double dDTSPf[]   = null; // Array of Time Difference S fast and P
  public double dDTSPs[]   = null; // Array of Time Difference S slow and P

  public double dGR[]      = null; // Array of Gamma Ray Log Values
  public double dCGR[]     = null; // Array of Gamma Ray minus Uranium Log Values

  public double dNPHI[]    = null; // Array of Neutron Porosity Log Values
  public double dPE[]      = null; // Array of PhotoElectric Log Values
  public double dRHOB[]    = null; // Array of Bulk Density Log Values
  public double dDPHI[]    = null; // Array of Density Porosity Log Values

  public double dDTC[]     = null; // Array of Sonic Acoustic Transit Log Values
  public double dDTS[]     = null; // Array of Sonic Shear Log Values
  public double dDTSF[]    = null; // Array of Sonic Fast Shear Log Values
  public double dDTSS[]    = null; // Array of Sonic Slow Shear Log Values
  public double dSPHI[]    = null; // Array of Sonic Porosity Log Values

  public double dTh[]      = null; // Array of Thorium Log Values
  public double dU[]       = null; // Array of Uranium Log Values
  public double dK[]       = null; // Array of Potassium Log Values

  // Rhomaa-Umaa Log Curves

  public double dUmaa[]    = null; // Apparent Volumetric Photoelectric factor
  public double dRhomaa[]  = null; // Apparent Matrix Grain Density
  public double dDTmaa[]   = null; // Apparent Matrix Grain Acoustic transit time

  // Alternate Lithology Curves

  public double dNeutron[] = null; // Neutron Porosity
  public double delta[]    = null; // Neutron - Density Porosity

  // Spectral Gamma Ray Lithology Curves

  public double dThU[]     = null; // Th/U
  public double dThK[]     = null; // Th/K

  // Primary Lithology

  public double dLITH[]    = null; // Primary Lithology

  // Sonic Computed Variables

  public double dVc[]      = null; // Compression Velocity (ft/sec)
  public double dVs[]      = null; // Compression Velocity (ft/sec)
  public double dVsf[]     = null; // Fast Shear Velocity (ft/sec)
  public double dVss[]     = null; // Slow Shear Velocity (ft/sec)

  public double dTc[]      = null; // Compression Time (milli-sec)
  public double dTs[]      = null; // Compression Time (milli-sec)
  public double dTsf[]     = null; // Fast Shear Time (milli-sec)
  public double dTss[]     = null; // Slow Shear Time (milli-sec)

  public double dAIc[]     = null; // Compression Acoustic Impedence (ft/sec * gm/cc)
  public double dAIs[]     = null; // Shear Acoustic Impedence (ft/sec * gm/cc)
  public double dAIsf[]    = null; // Fast Shear Acoustic Impedence (ft/sec * gm/cc)
  public double dAIss[]    = null; // Slow Shear Acoustic Impedence (ft/sec * gm/cc)

  public double dVAc[]     = null; // Average Compression Velocity (ft/sec)
  public double dVAs[]     = null; // Average Shear Velocity (ft/sec)

//  public String sTops[]    = null; // Tops assign to bed

  // Lithology Composition

  public int    iTotal     = 10;
  public int    iComp[][]  = null; // Lithology Composition

  /** Method delete()
   * <p> This method will set the arrays and strings to null to force Java to
   *     deallocate memory
   */

  public void delete()
  {
    iTOP      = -1;   // Top Depth
    iBASE     = -1;   // Base Depth

    iDTSP     = -1;   // Time Difference Shear and Compression
    iDTSPf    = -1;   // Time Difference Shear fast and Compression
    iDTSPs    = -1;   // Time Difference Shear slow and Compression

    iGR       = -1;   // Gamma Ray Curve Number in LAS File
    iCGR      = -1;   // Gamma Ray Minus Uranium Curve Number in LAS File

    iNPHI     = -1;   // Neutron Porosity Curve Number in LAS File
    iPE       = -1;   // Photoelectric Curve Number in LAS File
    iRHOB     = -1;   // Bulk Density Curve Number in LAS File
    iDPHI     = -1;   // Density Porosity Curve Number in LAS File

    iDTC      = -1;   // Sonic Compression Curve Number in LAS File
    iDTS      = -1;   // Sonic Shear Curve Number in LAS File
    iDTSF     = -1;   // Sonic Shear fast Curve Number in LAS File
    iDTSS     = -1;   // Sonic Shear slow Curve Number in LAS File
    iSPHI     = -1;   // Sonic Porisity Curve Number in LAS File

    iTh       = -1;   // Thorium Curve Number in LAS File
    iU        = -1;   // Uranium Curve Number in LAS File
    iK        = -1;   // Potasium Curve Number in LAS File

    // Computed Curves

    iUmaa     = -1;   // Apparent Volumetric Photoelectric factor
    iRhomaa   = -1;   // Apparent Matrix Grain Density
    iDTmaa    = -1;   // Apparent Matrix Grain Acoustic transit time

    // Alternate Lithology Curves

    iNeutron  = -1;  // Neutron Porosity
    iDelta    = -1;  // Neutron - Density Porosity

    // Spectral Gamma Ray Lithology Curves

    iThU      = -1;  // Th/U
    iThK      = -1;  // Th/K

    // Primary Lithology

    iLITH     = -1; // Lithology determined

    // Sonic Computed Variables

    iVc       = -1; // Compression Velocity (ft/sec)
    iVs       = -1; // Shear Velocity (ft/sec)
    iVsf      = -1; // Fast Shear Velocity (ft/sec)
    iVss      = -1; // Slow Shear Velocity (ft/sec)

    iTc       = -1; // Compression Time (milli-sec)
    iTs       = -1; // Shear Time (milli-sec)
    iTsf      = -1; // Fast Shear Time (milli-sec)
    iTss      = -1; // Slow Shear Time (milli-sec)

    iAIc      = -1; // Compression Acoustic Impedence (ft/sec * gm/cc)
    iAIs      = -1; // Shear Acoustic Impedence (ft/sec * gm/cc)
    iAIsf     = -1; // Fast Shear Acoustic Impedence (ft/sec * gm/cc)
    iAIss     = -1; // Slow Shear Acoustic Impedence (ft/sec * gm/cc)

    iVAc      = -1; // Average Compression Velocity (ft/sec)
    iVAs      = -1; // Average Shear Velocity (ft/sec)

    // Lithofacies Log Curves

    iRows     = 0;    // Number of Rows

    sName     = null; // Name of bed

    depths    = null; // Array of Depth Data Values
    dTime     = null; // Array of Time Data Values

    dTop      = null; // Array of Depth Start Data Values
    dBase     = null; // Array of Depth End Data Values

    dDTSP     = null; // Array of Time Difference Shear and Compression
    dDTSPf    = null; // Array of Time Difference Shear fast and Compression
    dDTSPs    = null; // Array of Time Difference Shear slow and Compression

    dGR       = null; // Array of Gamma Ray Log Values
    dCGR      = null; // Array of Gamma Ray minus Uranium Log Values

    dPE       = null; // Array of PhotoElectric Log Values
    dNPHI     = null; // Array of Neutron Porosity Log Values
    dRHOB     = null; // Array of Bulk Density Log Values
    dDPHI     = null; // Array of Density Porosity Log Values

    dDTC      = null; // Array of Sonic Log Values
    dDTS      = null; // Array of Sonic Shear Log Values
    dDTSF     = null; // Array of Sonic Shear fast Log Values
    dDTSS     = null; // Array of Sonic Shear slow Log Values
    dSPHI     = null; // Array of Sonic Porosity Log Values

    dTh       = null; // Array of Thorium Log Values
    dU        = null; // Array of Uranium Log Values
    dK        = null; // Array of Potassium Log Values

      // Rhomaa-Umaa Log Curves

    dUmaa     = null; // Apparent Volumetric Photoelectric factor
    dRhomaa   = null; // Apparent Matrix Grain Density
    dDTmaa    = null; // Apparent Matrix Grain Acoustic transit time

      // Alternate Lithology Curves

    dNeutron  = null; // Neutron Porosity
    delta     = null; // Neutron - Density Porosity

    // Spectral Gamma Ray Lithology Curves

    dThU      = null; // Th/U
    dThK      = null; // Th/K

    // Primary Lithology

    dLITH     = null; // Primary Lithology

  // Sonic Computed Variables

    dVc       = null; // Compression Velocity (ft/sec)
    dVs       = null; // Shear Velocity (ft/sec)
    dVsf      = null; // Fast Shear Velocity (ft/sec)
    dVss      = null; // Slow Shear Velocity (ft/sec)

    dTc       = null; // Compression Time (milli-sec)
    dTs       = null; // Shear Time (milli-sec)
    dTsf      = null; // Fast Shear Time (milli-sec)
    dTss      = null; // Slow Shear Time (milli-sec)

    dAIc      = null; // Compression Acoustic Impedence (ft/sec * gm/cc)
    dAIs      = null; // Shear Acoustic Impedence (ft/sec * gm/cc)
    dAIsf     = null; // Fast Shear Acoustic Impedence (ft/sec * gm/cc)
    dAIss     = null; // Slow Shear Acoustic Impedence (ft/sec * gm/cc)

    dVAc      = null; // Average Compression Velocity (ft/sec)
    dVAs      = null; // Average Shear Velocity (ft/sec)

//    sTops     = null; // Tops assign to bed

    // Lithology Composition

    iTotal     = 10;
    iComp      = null; // Lithology Composition
  }

  /** Method checkData(int iCurve)
   * <p> This method will check to see if the curve data is present
   * @param  iCurve = the requested curve type
   * @return bvalue = true data exist, false data does not exist
   */

  public boolean checkData(int iCurve)
  {
    boolean bValue = false;
    int     data   = -1;

    switch (iCurve)
    {
      case event.eventStandardTools._TOP:
        data = iTOP;
        break;
      case event.eventStandardTools._BASE:
        data = iBASE;
        break;

      case event.eventStandardTools._GR:
        data = iGR;
        break;
      case event.eventStandardTools._CGR:
        data = iCGR;
        break;

      case event.eventStandardTools._PE:
        data = iPE;
        break;
      case event.eventStandardTools._RHOB:
        data = iRHOB;
        break;
      case event.eventStandardTools._DPHI:
        data = iDPHI;
        break;
      case event.eventStandardTools._NPHI:
        data = iNPHI;
        break;

      case event.eventStandardTools._DTC:
        data = iDTC;
        break;
      case event.eventStandardTools._DTS:
        data = iDTS;
        break;
      case event.eventStandardTools._DTSF:
        data = iDTSF;
        break;
      case event.eventStandardTools._DTSS:
        data = iDTSS;
        break;
      case event.eventStandardTools._SPHI:
        data = iSPHI;
        break;

      case event.eventStandardTools._TH:
        data = iTh;
        break;
      case event.eventStandardTools._U:
        data = iU;
        break;
      case event.eventStandardTools._K:
        data = iK;
        break;

      case event.eventStandardTools._RHOMAA:
        data = iRhomaa;
        break;
      case event.eventStandardTools._UMAA:
        data = iUmaa;
        break;
      case event.eventStandardTools._DTMAA:
        data = iDTmaa;
        break;

      case event.eventStandardTools._PHIDIFF:
        data = iDelta;
        break;

      case event.eventStandardTools._THU:
        data = iThU;
        break;
      case event.eventStandardTools._THK:
        data = iThK;
        break;

      case event.eventStandardTools._LITH:
        data = iLITH;
        break;

      case event.eventStandardTools._Vc:
        data = iVc;
        break;
      case event.eventStandardTools._Vs:
        data = iVs;
        break;
      case event.eventStandardTools._Vsf:
        data = iVsf;
        break;
      case event.eventStandardTools._Vss:
        data = iVss;
        break;

      case event.eventStandardTools._VAc:
        data = iVAc;
        break;
      case event.eventStandardTools._VAs:
        data = iVAs;
        break;

      case event.eventStandardTools._Tc:
        data = iTc;
        break;
      case event.eventStandardTools._Ts:
        data = iTs;
        break;
      case event.eventStandardTools._Tsf:
        data = iTsf;
        break;
      case event.eventStandardTools._Tss:
        data = iTss;
        break;

      case event.eventStandardTools._AIc:
        data = iAIc;
        break;
      case event.eventStandardTools._AIs:
        data = iAIs;
        break;
      case event.eventStandardTools._AIsf:
        data = iAIsf;
        break;
      case event.eventStandardTools._AIss:
        data = iAIss;
        break;

      case event.eventStandardTools._TSP:
        data = iDTSP;
        break;
      case event.eventStandardTools._TSPf:
        data = iDTSPf;
        break;
      case event.eventStandardTools._TSPs:
        data = iDTSPs;
        break;
	}

    if (data > -1)
      bValue = true;

    return (bValue);
  }

  /** Method getData(int iCurve)
   * <p> This method will retrive the curve data requested
   * @param  iCurve = the requested curve type
   * @return data   = the data requested.
   */

  public double[] getData(int iCurve)
  {
    double data[] = null;

    if (iRows > 0)
    {
      data = new double[iRows];

      switch (iCurve)
      {
        case event.eventStandardTools._TOP:
          data = dTop;
          break;
        case event.eventStandardTools._BASE:
          data = dBase;
          break;

        case event.eventStandardTools._GR:
          data = dGR;
          break;
        case event.eventStandardTools._CGR:
          data = dCGR;
          break;

        case event.eventStandardTools._PE:
          data = dPE;
          break;
        case event.eventStandardTools._RHOB:
          data = dRHOB;
          break;
        case event.eventStandardTools._DPHI:
          data = dDPHI;
          break;
        case event.eventStandardTools._NPHI:
          data = dNPHI;
          break;

        case event.eventStandardTools._DTC:
          data = dDTC;
          break;
        case event.eventStandardTools._DTS:
          data = dDTS;
          break;
        case event.eventStandardTools._DTSF:
          data = dDTSF;
          break;
        case event.eventStandardTools._DTSS:
          data = dDTSS;
          break;
        case event.eventStandardTools._SPHI:
          data = dSPHI;
          break;

        case event.eventStandardTools._TH:
          data = dTh;
          break;
        case event.eventStandardTools._U:
          data = dU;
          break;
        case event.eventStandardTools._K:
          data = dK;
          break;

        case event.eventStandardTools._RHOMAA:
          data = dRhomaa;
          break;
        case event.eventStandardTools._UMAA:
          data = dUmaa;
          break;
        case event.eventStandardTools._DTMAA:
          data = dDTmaa;
          break;
        case event.eventStandardTools._PHIDIFF:
          data = delta;
          break;

        case event.eventStandardTools._THU:
          data = dThU;
          break;
        case event.eventStandardTools._THK:
          data = dThK;
          break;

        case event.eventStandardTools._LITH:
          data = dLITH;
          break;

        case event.eventStandardTools._Vc:
          data = dVc;
          break;
        case event.eventStandardTools._Vs:
          data = dVs;
          break;
        case event.eventStandardTools._Vsf:
          data = dVsf;
          break;
        case event.eventStandardTools._Vss:
          data = dVss;
          break;

        case event.eventStandardTools._VAc:
          data = dVAc;
          break;
        case event.eventStandardTools._VAs:
          data = dVAs;
          break;

        case event.eventStandardTools._Tc:
          data = dTc;
          break;
        case event.eventStandardTools._Ts:
          data = dTs;
          break;
        case event.eventStandardTools._Tsf:
          data = dTsf;
          break;
        case event.eventStandardTools._Tss:
          data = dTss;
          break;

        case event.eventStandardTools._AIc:
          data = dAIc;
          break;
        case event.eventStandardTools._AIs:
          data = dAIs;
          break;
        case event.eventStandardTools._AIsf:
          data = dAIsf;
          break;
        case event.eventStandardTools._AIss:
          data = dAIss;
          break;

        case event.eventStandardTools._TSP:
          data = dDTSP;
          break;
        case event.eventStandardTools._TSPf:
          data = dDTSPf;
          break;
        case event.eventStandardTools._TSPs:
          data = dDTSPs;
          break;
      }
    }

    return (data);
  }

  /** Method getRow(int iCurve, int iRow)
   * <p> This method will retrive the curve data requested
   * @param  iCurve  = the requested curve type
   * @param  iRow    = The LAS Row
   * @return data    = the data requested.
   */

  public double getRow(int iCurve, int iRow)
  {
    double data = dNull;

    if (iRows > 0)
    {
      switch (iCurve)
      {
        case event.eventStandardTools._TOP:
          data = dTop[iRow];
          break;
        case event.eventStandardTools._BASE:
          data = dBase[iRow];
          break;

        case event.eventStandardTools._GR:
          data = dGR[iRow];
          break;
        case event.eventStandardTools._CGR:
          data = dCGR[iRow];
          break;

        case event.eventStandardTools._PE:
          data = dPE[iRow];
          break;
        case event.eventStandardTools._RHOB:
          data = dRHOB[iRow];
          break;
        case event.eventStandardTools._DPHI:
          data = dDPHI[iRow];
          break;
        case event.eventStandardTools._NPHI:
          data = dNPHI[iRow];
          break;

        case event.eventStandardTools._DTC:
          data = dDTC[iRow];
          break;
        case event.eventStandardTools._DTS:
          data = dDTS[iRow];
          break;
        case event.eventStandardTools._DTSF:
          data = dDTSF[iRow];
          break;
        case event.eventStandardTools._DTSS:
          data = dDTSS[iRow];
          break;
        case event.eventStandardTools._SPHI:
          data = dSPHI[iRow];
          break;

        case event.eventStandardTools._TH:
          data = dTh[iRow];
          break;
        case event.eventStandardTools._U:
          data = dU[iRow];
          break;
        case event.eventStandardTools._K:
          data = dK[iRow];
          break;

        case event.eventStandardTools._RHOMAA:
          data = dRhomaa[iRow];
          break;
        case event.eventStandardTools._UMAA:
          data = dUmaa[iRow];
          break;
        case event.eventStandardTools._DTMAA:
          data = dDTmaa[iRow];
          break;
        case event.eventStandardTools._PHIDIFF:
          data = delta[iRow];
          break;

        case event.eventStandardTools._THU:
          data = dThU[iRow];
          break;
        case event.eventStandardTools._THK:
          data = dThK[iRow];
          break;

        case event.eventStandardTools._LITH:
          data = dLITH[iRow];
          break;

        case event.eventStandardTools._Vc:
          data = dVc[iRow];
          break;
        case event.eventStandardTools._Vs:
          data = dVs[iRow];
          break;
        case event.eventStandardTools._Vsf:
          data = dVsf[iRow];
          break;
        case event.eventStandardTools._Vss:
          data = dVss[iRow];
          break;

        case event.eventStandardTools._VAc:
          data = dVAc[iRow];
          break;
        case event.eventStandardTools._VAs:
          data = dVAs[iRow];
          break;

        case event.eventStandardTools._Tc:
          data = dTc[iRow];
          break;
        case event.eventStandardTools._Ts:
          data = dTs[iRow];
          break;
        case event.eventStandardTools._Tsf:
          data = dTsf[iRow];
          break;
        case event.eventStandardTools._Tss:
          data = dTss[iRow];
          break;

        case event.eventStandardTools._AIc:
          data = dAIc[iRow];
          break;
        case event.eventStandardTools._AIs:
          data = dAIs[iRow];
          break;
        case event.eventStandardTools._AIsf:
          data = dAIsf[iRow];
          break;
        case event.eventStandardTools._AIss:
          data = dAIss[iRow];
          break;

        case event.eventStandardTools._TSP:
          data = dDTSP[iRow];
          break;
        case event.eventStandardTools._TSPf:
          data = dDTSPf[iRow];
          break;
        case event.eventStandardTools._TSPs:
          data = dDTSPs[iRow];
          break;
      }
    }

    return (data);
  }

  /** Method setData(int iCurve, int iColumn)
   * <p> This method will check to see if the curve data is present
   * @param  iCurve  = the requested curve type
   * @param  iColumn = the column number for rock data
   */

  public void setData(int iCurve, int iColumn)
  {
    switch (iCurve)
    {
      case event.eventStandardTools._TOP:
        iTOP = iColumn;
        break;
      case event.eventStandardTools._BASE:
        iBASE = iColumn;
        break;

      case event.eventStandardTools._GR:
        iGR = iColumn;
        break;
      case event.eventStandardTools._CGR:
        iCGR = iColumn;
        break;

      case event.eventStandardTools._PE:
        iPE = iColumn;
        break;
      case event.eventStandardTools._RHOB:
        iRHOB = iColumn;
        break;
      case event.eventStandardTools._DPHI:
        iDPHI = iColumn;
        break;
      case event.eventStandardTools._NPHI:
        iNPHI = iColumn;
        break;

      case event.eventStandardTools._DTC:
        iDTC = iColumn;
        break;
      case event.eventStandardTools._DTS:
        iDTS = iColumn;
        break;
      case event.eventStandardTools._DTSF:
        iDTSF = iColumn;
        break;
      case event.eventStandardTools._DTSS:
        iDTSS = iColumn;
        break;
      case event.eventStandardTools._SPHI:
        iSPHI = iColumn;
        break;

      case event.eventStandardTools._TH:
        iTh = iColumn;
        break;
      case event.eventStandardTools._U:
        iU = iColumn;
        break;
      case event.eventStandardTools._K:
        iK = iColumn;
        break;

      case event.eventStandardTools._RHOMAA:
        iRhomaa = iColumn;
        break;
      case event.eventStandardTools._UMAA:
        iUmaa = iColumn;
        break;
      case event.eventStandardTools._PHIDIFF:
        iDelta = iColumn;
        break;

      case event.eventStandardTools._THU:
        iThU = iColumn;
        break;
      case event.eventStandardTools._THK:
        iThK = iColumn;
        break;

      case event.eventStandardTools._LITH:
        iLITH = iColumn;
        break;

      case event.eventStandardTools._Vc:
        iVc = iColumn;
        break;
      case event.eventStandardTools._Vs:
        iVs = iColumn;
        break;
      case event.eventStandardTools._Vsf:
        iVsf = iColumn;
        break;
      case event.eventStandardTools._Vss:
        iVss = iColumn;
        break;

      case event.eventStandardTools._VAc:
        iVAc = iColumn;
        break;
      case event.eventStandardTools._VAs:
        iVAs = iColumn;
        break;

      case event.eventStandardTools._Tc:
        iTc = iColumn;
        break;
      case event.eventStandardTools._Ts:
        iTs = iColumn;
        break;
      case event.eventStandardTools._Tsf:
        iTsf = iColumn;
        break;
      case event.eventStandardTools._Tss:
        iTss = iColumn;
        break;

      case event.eventStandardTools._AIc:
        iAIc = iColumn;
        break;
      case event.eventStandardTools._AIs:
        iAIs = iColumn;
        break;
      case event.eventStandardTools._AIsf:
        iAIsf = iColumn;
        break;
      case event.eventStandardTools._AIss:
        iAIss = iColumn;
        break;

      case event.eventStandardTools._TSP:
        iDTSP = iColumn;
        break;
      case event.eventStandardTools._TSPf:
        iDTSPf = iColumn;
        break;
      case event.eventStandardTools._TSPs:
        iDTSPs = iColumn;
        break;
    }
  }

  /** Method setRow(int iCurve, int iRow, double data)
   * <p> This method will set the curve data requested
   * @param  iCurve  = the requested curve type
   * @param  iRow    = The LAS Row
   * @param  data    = the data being modified.
   */

  public void setRow(int iCurve, int iRow, double data)
  {
    if (iRows > 0)
    {
      switch (iCurve)
      {
        case event.eventStandardTools._TOP:
          dTop[iRow] = data;
          break;
        case event.eventStandardTools._BASE:
          dBase[iRow] = data;
          break;

        case event.eventStandardTools._GR:
          dGR[iRow] = data;
          break;
        case event.eventStandardTools._CGR:
          dCGR[iRow] = data;
          break;

        case event.eventStandardTools._PE:
          dPE[iRow] = data;
          break;
        case event.eventStandardTools._RHOB:
          dRHOB[iRow] = data;
          break;
        case event.eventStandardTools._DPHI:
          dDPHI[iRow] = data;
          break;
        case event.eventStandardTools._NPHI:
          dNPHI[iRow] = data;
          break;

        case event.eventStandardTools._DTC:
          dDTC[iRow] = data;
          break;
        case event.eventStandardTools._DTS:
          dDTS[iRow] = data;
          break;
        case event.eventStandardTools._DTSF:
          dDTSF[iRow] = data;
          break;
        case event.eventStandardTools._DTSS:
          dDTSS[iRow] = data;
          break;
        case event.eventStandardTools._SPHI:
          dSPHI[iRow] = data;
          break;

        case event.eventStandardTools._TH:
          dTh[iRow] = data;
          break;
        case event.eventStandardTools._U:
          dU[iRow] = data;
          break;
        case event.eventStandardTools._K:
          dK[iRow] = data;
          break;

        case event.eventStandardTools._RHOMAA:
          dRhomaa[iRow] = data;
          break;
        case event.eventStandardTools._UMAA:
          dUmaa[iRow] = data;
          break;
        case event.eventStandardTools._DTMAA:
          dDTmaa[iRow] = data;
          break;
        case event.eventStandardTools._PHIDIFF:
          delta[iRow] = data;
          break;

        case event.eventStandardTools._THU:
          dThU[iRow] = data;
          break;
        case event.eventStandardTools._THK:
          dThK[iRow] = data;
          break;

        case event.eventStandardTools._LITH:
          dLITH[iRow] = data;
          break;

        case event.eventStandardTools._Vc:
          dVc[iRow] = data;
          break;
        case event.eventStandardTools._Vs:
          dVs[iRow] = data;
          break;
        case event.eventStandardTools._Vsf:
          dVsf[iRow] = data;
          break;
        case event.eventStandardTools._Vss:
          dVss[iRow] = data;
          break;

        case event.eventStandardTools._VAc:
          dVAc[iRow] = data;
          break;
        case event.eventStandardTools._VAs:
          dVAs[iRow] = data;
          break;

        case event.eventStandardTools._Tc:
          dTc[iRow] = data;
          break;
        case event.eventStandardTools._Ts:
          dTs[iRow] = data;
          break;
        case event.eventStandardTools._Tsf:
          dTsf[iRow] = data;
          break;
        case event.eventStandardTools._Tss:
          dTss[iRow] = data;
          break;

        case event.eventStandardTools._AIc:
          dAIc[iRow] = data;
          break;
        case event.eventStandardTools._AIs:
          dAIs[iRow] = data;
          break;
        case event.eventStandardTools._AIsf:
          dAIsf[iRow] = data;
          break;
        case event.eventStandardTools._AIss:
          dAIss[iRow] = data;
          break;

        case event.eventStandardTools._TSP:
          dDTSP[iRow] = data;
          break;
        case event.eventStandardTools._TSPf:
          dDTSPf[iRow] = data;
          break;
        case event.eventStandardTools._TSPs:
          dDTSPs[iRow] = data;
          break;
      }
    }
  }
}

/*
 *  @version 1.1 02/18/2015
 *  @author  John Victorine
 */
