/*
 * @eventFileDataUtility.java Version 1.1 02/19/2015
 *
 * Copyright (c) 2015 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package event;

import event.eventFileDataStruct;

import lith.lithology.lithologySymbolsListStruct;

import horizon.strat.stratStruct;
import horizon.strat.stratListStruct;

/** Class eventFileDataUtility
 *  <p> This Class will provide basic utilities for the LAS Curves
 *      data structures.
 *
 *  @version 1.1 02/19/2015
 *  @author  John R. Victorine
 */

public class eventFileDataUtility
{
  /** Method initData()
   * <p> This method will initialize the LAS File data
   * @param  iRows = total number of rows
   * @return st    = Seismic Data Structure
   */

  public static eventFileDataStruct initData(int iRows)
  {
    eventFileDataStruct st    = new eventFileDataStruct();
    double              dNull = st.dNull;  // Null Value

    if (iRows > 0)
    {
      st.iRows     = iRows;                // Number of Rows

      st.sName     = new String[st.iRows]; // Name of bed

      st.depths    = new double[st.iRows]; // Array of Depth Data Values
      st.dTime     = new double[st.iRows]; // Array of Time Data Values

      st.dTop      = new double[st.iRows]; // Array of Depth Start Data Values
      st.dBase     = new double[st.iRows]; // Array of Depth End Data Values

      st.dDTSP     = new double[st.iRows]; // Time Difference Shear and Compression
      st.dDTSPf    = new double[st.iRows]; // Time Difference Shear fast and Compression
      st.dDTSPs    = new double[st.iRows]; // Time Difference Shear slow and Compression

      st.dGR       = new double[st.iRows]; // Array of Gamma Ray Log Values
      st.dCGR      = new double[st.iRows]; // Array of Gamma Ray minus Uranium Log Values

      st.dGR       = new double[st.iRows]; // Array of Gamma Ray Log Values
      st.dCGR      = new double[st.iRows]; // Array of Gamma Ray minus Uranium Log Values

      st.dNPHI     = new double[st.iRows]; // Array of Neutron Porosity Log Values
      st.dPE       = new double[st.iRows]; // Array of PhotoElectric  Values
      st.dRHOB     = new double[st.iRows]; // Array of Bulk Density  Values
      st.dDPHI     = new double[st.iRows]; // Array of Density Porosity Log Values

      st.dDTC      = new double[st.iRows]; // Array of Sonic Acoustic Transit Time Log Values
      st.dDTS      = new double[st.iRows]; // Array of Sonic Shear Log Values
      st.dDTSF     = new double[st.iRows]; // Array of Sonic Fast Shear Log Values
      st.dDTSS     = new double[st.iRows]; // Array of Sonic Slow Shear Log Values
      st.dSPHI     = new double[st.iRows]; // Array of Sonic Porosity Log Values

      st.dTh       = new double[st.iRows]; // Array of Thorium Log Values
      st.dU        = new double[st.iRows]; // Array of Uranium Log Values
      st.dK        = new double[st.iRows]; // Array of Potassium Log Values

      st.dUmaa     = new double[st.iRows]; // Apparent Volumetric Photoelectric
      st.dRhomaa   = new double[st.iRows]; // Apparent Matrix Grain Density
      st.dDTmaa    = new double[st.iRows]; // Apparent Matrix Grain Acoustic transit time

      st.dNeutron  = new double[st.iRows]; // Neutron Porosity
      st.delta     = new double[st.iRows]; // Neutron - Density Porosity

      st.dThU      = new double[st.iRows]; // Th/U
      st.dThK      = new double[st.iRows]; // Th/K

      st.dLITH     = new double[st.iRows]; // Primary Lithology Indicator

      // Sonic Computed Variables

      st.dVc       = new double[st.iRows]; // Compression Velocity (ft/sec)
      st.dVs       = new double[st.iRows]; // Shear Velocity (ft/sec)
      st.dVsf      = new double[st.iRows]; // Fast Shear Velocity (ft/sec)
      st.dVss      = new double[st.iRows]; // Slow Shear Velocity (ft/sec)

      st.dVAc      = new double[st.iRows]; // Average Compression Velocity (ft/sec)
      st.dVAs      = new double[st.iRows]; // Average Shear Velocity (ft/sec)

      st.dTc       = new double[st.iRows]; // Compression Time (milli-sec)
      st.dTs       = new double[st.iRows]; // Shear Time (milli-sec)
      st.dTsf      = new double[st.iRows]; // Fast Shear Time (milli-sec)
      st.dTss      = new double[st.iRows]; // Slow Shear Time (milli-sec)

      st.dAIc      = new double[st.iRows]; // Compression Acoustic Impedence (ft/sec * gm/cc)
      st.dAIs      = new double[st.iRows]; // Shear Acoustic Impedence (ft/sec * gm/cc)
      st.dAIsf     = new double[st.iRows]; // Fast Shear Acoustic Impedence (ft/sec * gm/cc)
      st.dAIss     = new double[st.iRows]; // Slow Shear Acoustic Impedence (ft/sec * gm/cc)

//      st.sTops     = new String[st.iRows]; // Name assign to bed

      st.iTotal    = 10;
      st.iComp     = new int[st.iRows][st.iTotal]; // Lithology Composition

      for (int i=0; i<st.iRows; i++)
      {
		st.sName[i]     = new String(""); // Array of Bed names

        st.depths[i]    = dNull; // Array of Depth Data Values
        st.dTime[i]     = dNull; // Array of Time Data Values

        st.dTop[i]      = dNull; // Array of Depth Start Data Values
        st.dBase[i]     = dNull; // Array of Depth End Data Values

        st.dDTSP[i]     = dNull; // Time Difference Shear and Compression
        st.dDTSPf[i]    = dNull; // Time Difference Shear fast and Compression
        st.dDTSPs[i]    = dNull; // Time Difference Shear slow and Compression

        st.dGR[i]       = dNull; // Array of Gamma Ray Log Values
        st.dCGR[i]      = dNull; // Array of Gamma Ray minus Uranium Log Values

        st.dPE[i]       = dNull; // Array of PhotoElectric Log Values
        st.dNPHI[i]     = dNull; // Array of Neutron Porosity Log Values
        st.dRHOB[i]     = dNull; // Array of Bulk Density Log Values
        st.dDPHI[i]     = dNull; // Array of Density Porosity Log Values

        st.dDTC[i]      = dNull; // Array of Sonic Acoustic Transit Time Log Values
        st.dDTS[i]      = dNull; // Array of Sonic Shear Log Values
        st.dDTSF[i]     = dNull; // Array of Sonic Fast Shear Log Values
        st.dDTSS[i]     = dNull; // Array of Sonic Slow Shear Log Values
        st.dSPHI[i]     = dNull; // Array of Sonic Porosity Log Values

        st.dTh[i]       = dNull; // Array of Thorium Log Values
        st.dU[i]        = dNull; // Array of Uranium Log Values
        st.dK[i]        = dNull; // Array of Potassium Log Values

        // Rhomaa-Umaa Log Curves

        st.dUmaa[i]     = dNull; // Apparent Volumetric Photoelectric
        st.dRhomaa[i]   = dNull; // Apparent Matrix Grain Density
        st.dDTmaa[i]    = dNull; // Apparent Matrix Grain Acoustic transit time

        // Alternate Lithology Curves

        st.dNeutron[i]  = dNull; // Neutron Porosity
        st.delta[i]     = dNull; // Neutron - Density Porosity

        // Spectral Gamma Ray Lithology Curves

        st.dThU[i]      = dNull; // Th/U
        st.dThK[i]      = dNull; // Th/K

        // Primary Lithology

        st.dLITH[i]     = -1; // Primary Lithology

        // Sonic Computed Variables

        st.dVc[i]       = dNull; // Compression Velocity (ft/sec)
        st.dVs[i]       = dNull; // Shear Velocity (ft/sec)
        st.dVsf[i]      = dNull; // Fast Shear Velocity (ft/sec)
        st.dVss[i]      = dNull; // Slow Shear Velocity (ft/sec)

        st.dVAc[i]      = dNull; // Average Compression Velocity (ft/sec)
        st.dVAs[i]      = dNull; // Average Shear Velocity (ft/sec)

        st.dTc[i]       = dNull; // Compression Time (milli-sec)
        st.dTs[i]       = dNull; // Shear Time (milli-sec)
        st.dTsf[i]      = dNull; // Fast Shear Time (milli-sec)
        st.dTss[i]      = dNull; // Slow Shear Time (milli-sec)

        st.dAIc[i]      = dNull; // Compression Acoustic Impedence (ft/sec * gm/cc)
        st.dAIs[i]      = dNull; // Shear Acoustic Impedence (ft/sec * gm/cc)
        st.dAIsf[i]     = dNull; // Fast Shear Acoustic Impedence (ft/sec * gm/cc)
        st.dAIss[i]     = dNull; // Slow Shear Acoustic Impedence (ft/sec * gm/cc)

//        st.sTops[i]     = new String(""); // Name assign to bed

        for (int j=0; j<st.iTotal; j++)
          st.iComp[i][j] = 0;
	  }
	}

	return (st);
  }

  /** Method copy()
   * <p> This method will copy the LAS Data from one file to another.
   * @param   stOld = Old Data Structure
   * @return  stNew = New Data Structure
   */

  public static eventFileDataStruct copy(eventFileDataStruct stOld)
  {
    eventFileDataStruct stNew = null;

    if (stOld != null)
    {
      stNew           = new eventFileDataStruct();

      stNew.dNull     = stOld.dNull;  // Default Null Value of Data
      stNew.dStart    = stOld.dStart; // Starting Depth Value of Data
      stNew.dEnd      = stOld.dEnd;   // Ending Depth Value of Data

      stNew.dElev     = stOld.dElev;  // Elevation of the Data Set

      // Curves Available

      stNew.iTOP      = stOld.iTOP;      // Top Depth
      stNew.iBASE     = stOld.iBASE;     // Base Depth

      stNew.iDTSP     = stOld.iDTSP;     // Time Difference Shear and Compression
      stNew.iDTSPf    = stOld.iDTSPf;    // Time Difference Shear fast and Compression
      stNew.iDTSPs    = stOld.iDTSPs;    // Time Difference Shear slow and Compression

      stNew.iGR       = stOld.iGR;       // Gamma Ray Curve Number in LAS File
      stNew.iCGR      = stOld.iCGR;      // Gamma Ray Minus Uranium Curve Number

      stNew.iNPHI     = stOld.iNPHI;     // Neutron Porosity Curve Number in LAS File
      stNew.iPE       = stOld.iPE;       // Photoelectric Curve Number in LAS File
      stNew.iRHOB     = stOld.iRHOB;     // Bulk Density Curve Number in LAS File
      stNew.iDPHI     = stOld.iDPHI;     // Density Porosity Curve Number in LAS File

      stNew.iDTC      = stOld.iDTC;      // Sonic Acoustic Transit Time Curve Number in LAS File
      stNew.iDTS      = stOld.iDTS;      // Sonic Fast Curve Number in LAS File
      stNew.iDTSF     = stOld.iDTSF;     // Sonic Fast Shear Curve Number in LAS File
      stNew.iDTSS     = stOld.iDTSS;     // Sonic Slow Shear Curve Number in LAS File
      stNew.iSPHI     = stOld.iSPHI;     // Sonic Porisity Curve Number in LAS File

      stNew.iTh       = stOld.iTh;       // Thorium Curve Number in LAS File
      stNew.iU        = stOld.iU;        // Uranium Curve Number in LAS File
      stNew.iK        = stOld.iK;        // Potasium Curve Number in LAS File

      stNew.iUmaa     = stOld.iUmaa;     // Apparent Volumetric Photoelectric factor
      stNew.iRhomaa   = stOld.iRhomaa;   // Apparent Matrix Grain Density
      stNew.iDTmaa    = stOld.iDTmaa;    // Apparent Matrix Grain Acoustic transit time

        // Alternate Lithology Curves

      stNew.iNeutron  = stOld.iNeutron;  // Neutron Porosity
      stNew.iDelta    = stOld.iDelta;    // Neutron - Density Porosity

      // Spectral Gamma Ray Lithology Curves

      stNew.iThU      = stOld.iThU;      // Th/U
      stNew.iThK      = stOld.iThK;      // Th/K

      // Primary Lithology

      stNew.iLITH     = stOld.iLITH;     // Lithology determined

      // Sonic Computed Variables

      stNew.iVc       = stOld.iVc;   // Compression Velocity (ft/sec)
      stNew.iVs       = stOld.iVs;   // Shear Velocity (ft/sec)
      stNew.iVsf      = stOld.iVsf;  // Fast Shear Velocity (ft/sec)
      stNew.iVss      = stOld.iVss;  // Slow Shear Velocity (ft/sec)

      stNew.iVAc      = stOld.iVAc;  // Average Compression Velocity (ft/sec)
      stNew.iVAs      = stOld.iVAs;  // Average Shear Velocity (ft/sec)

      stNew.iTc       = stOld.iTc;   // Compression Time (milli-sec)
      stNew.iTs       = stOld.iTs;   // Shear Time (milli-sec)
      stNew.iTsf      = stOld.iTsf;  // Fast Shear Time (milli-sec)
      stNew.iTss      = stOld.iTss;  // Slow Shear Time (milli-sec)

      stNew.iAIc      = stOld.iAIc;  // Compression Acoustic Impedence (ft/sec * gm/cc)
      stNew.iAIs      = stOld.iAIs;  // Shear Acoustic Impedence (ft/sec * gm/cc)
      stNew.iAIsf     = stOld.iAIsf; // Fast Shear Acoustic Impedence (ft/sec * gm/cc)
      stNew.iAIss     = stOld.iAIss; // Slow Shear Acoustic Impedence (ft/sec * gm/cc)

//      stNew.iTSP      = stOld.iTSP;  // S-P Time Difference (milli-sec)
//      stNew.iTSPf     = stOld.iTSPf; // Sf-P Time Difference (milli-sec)
//      stNew.iTSPs     = stOld.iTSPs; // Ss-P Time Difference (milli-sec)

      // Lithofacies Log Curves

      stNew.iRows     = stOld.iRows;    // Number of Rows

      stNew.sName     = new String[stNew.iRows]; // Name of bed

      stNew.depths    = new double[stNew.iRows]; // Array of Depth Data Values
      stNew.dTime     = new double[stNew.iRows]; // Array of Time Data Values

      stNew.dTop      = new double[stNew.iRows]; // Array of Depth Start Data Values
      stNew.dBase     = new double[stNew.iRows]; // Array of Depth End Data Values

      stNew.dDTSP     = new double[stNew.iRows]; // Time Difference Shear and Compression
      stNew.dDTSPf    = new double[stNew.iRows]; // Time Difference Shear fast and Compression
      stNew.dDTSPs    = new double[stNew.iRows]; // Time Difference Shear slow and Compression

      stNew.dGR       = new double[stNew.iRows]; // Array of Gamma Ray Log Values
      stNew.dCGR      = new double[stNew.iRows]; // Array of Gamma Ray minus Uranium Log Values

      stNew.dNPHI     = new double[stNew.iRows]; // Array of Neutron Porosity Log Values
      stNew.dPE       = new double[stNew.iRows]; // Array of PhotoElectric  Values
      stNew.dRHOB     = new double[stNew.iRows]; // Array of Bulk Density  Values
      stNew.dDPHI     = new double[stNew.iRows]; // Array of Density Porosity Log Values

      stNew.dDTC      = new double[stNew.iRows]; // Array of Sonic Acoustic Transit Time Log Values
      stNew.dDTS      = new double[stNew.iRows]; // Array of Sonic Shear Log Values
      stNew.dDTSF     = new double[stNew.iRows]; // Array of Sonic Fast Shear Log Values
      stNew.dDTSS     = new double[stNew.iRows]; // Array of Sonic Slow Shear Log Values
      stNew.dSPHI     = new double[stNew.iRows]; // Array of Sonic Porosity Log Values

      stNew.dTh       = new double[stNew.iRows]; // Array of Thorium Log Values
      stNew.dU        = new double[stNew.iRows]; // Array of Uranium Log Values
      stNew.dK        = new double[stNew.iRows]; // Array of Potassium Log Values

      stNew.dUmaa     = new double[stNew.iRows]; // Apparent Volumetric Photoelectric
      stNew.dRhomaa   = new double[stNew.iRows]; // Apparent Matrix Grain Density
      stNew.dDTmaa    = new double[stNew.iRows]; // Apparent Matrix Grain Acoustic transit time

      stNew.dNeutron  = new double[stNew.iRows]; // Neutron Porosity
      stNew.delta     = new double[stNew.iRows]; // Neutron - Density Porosity

      stNew.dThU      = new double[stNew.iRows]; // Th/U
      stNew.dThK      = new double[stNew.iRows]; // Th/K

      stNew.dLITH     = new double[stNew.iRows]; // Primary Lithology Indicator

      // Sonic Computed Variables

      stNew.dVc       = new double[stNew.iRows]; // Compression Velocity (ft/sec)
      stNew.dVs       = new double[stNew.iRows]; // Shear Velocity (ft/sec)
      stNew.dVsf      = new double[stNew.iRows]; // Fast Shear Velocity (ft/sec)
      stNew.dVss      = new double[stNew.iRows]; // Slow Shear Velocity (ft/sec)

      stNew.dVAc      = new double[stNew.iRows]; // Average Compression Velocity (ft/sec)
      stNew.dVAs      = new double[stNew.iRows]; // Average Shear Velocity (ft/sec)

      stNew.dTc       = new double[stNew.iRows]; // Compression Time (milli-sec)
      stNew.dTs       = new double[stNew.iRows]; // Shear Time (milli-sec)
      stNew.dTsf      = new double[stNew.iRows]; // Fast Shear Time (milli-sec)
      stNew.dTss      = new double[stNew.iRows]; // Slow Shear Time (milli-sec)

      stNew.dAIc      = new double[stNew.iRows]; // Compression Acoustic Impedence (ft/sec * gm/cc)
      stNew.dAIs      = new double[stNew.iRows]; // Shear Acoustic Impedence (ft/sec * gm/cc)
      stNew.dAIsf     = new double[stNew.iRows]; // Fast Shear Acoustic Impedence (ft/sec * gm/cc)
      stNew.dAIss     = new double[stNew.iRows]; // Slow Shear Acoustic Impedence (ft/sec * gm/cc)

//      stNew.sTops     = new String[stNew.iRows]; // Name assign to bed

      stNew.iTotal    = 10;
      stNew.iComp     = new int[stNew.iRows][stNew.iTotal]; // Lithology Composition

      for (int i=0; i<stOld.iRows; i++)
      {
		stNew.sName[i]     = new String(stOld.sName[i]); // Array of Bed names

        stNew.depths[i]    = stOld.depths[i];    // Array of Depth Data Values
        stNew.dTime[i]     = stOld.dTime[i];     // Array of Time Data Values

        stNew.dTop[i]      = stOld.dTop[i];      // Array of Depth Start Data Values
        stNew.dBase[i]     = stOld.dBase[i];     // Array of Depth End Data Values

        stNew.dDTSP[i]     = stOld.dDTSP[i];     // Time Difference Shear and Compression
        stNew.dDTSPf[i]    = stOld.dDTSPf[i];    // Time Difference Shear fast and Compression
        stNew.dDTSPs[i]    = stOld.dDTSPs[i];    // Time Difference Shear slow and Compression

        stNew.dGR[i]       = stOld.dGR[i];       // Array of Gamma Ray Log Values
        stNew.dCGR[i]      = stOld.dCGR[i];      // Array of Gamma Ray minus Uranium Log Values

        stNew.dPE[i]       = stOld.dPE[i];       // Array of PhotoElectric Log Values
        stNew.dNPHI[i]     = stOld.dNPHI[i];     // Array of Neutron Porosity Log Values
        stNew.dRHOB[i]     = stOld.dRHOB[i];     // Array of Bulk Density Log Values
        stNew.dDPHI[i]     = stOld.dDPHI[i];     // Array of Density Porosity Log Values

        stNew.dDTC[i]      = stOld.dDTC[i];      // Array of Sonic Acoustic Transit Time Log Values
        stNew.dDTS[i]      = stOld.dDTS[i];      // Array of Sonic Shear Log Values
        stNew.dDTSF[i]     = stOld.dDTSF[i];     // Array of Sonic Fast Shear Log Values
        stNew.dDTSS[i]     = stOld.dDTSS[i];     // Array of Sonic Slow Shear Log Values
        stNew.dSPHI[i]     = stOld.dSPHI[i];     // Array of Sonic Porosity Log Values

        stNew.dTh[i]       = stOld.dTh[i];       // Array of Thorium Log Values
        stNew.dU[i]        = stOld.dU[i];        // Array of Uranium Log Values
        stNew.dK[i]        = stOld.dK[i];        // Array of Potassium Log Values

        // Rhomaa-Umaa Log Curves

        stNew.dUmaa[i]     = stOld.dUmaa[i];     // Apparent Volumetric Photoelectric
        stNew.dRhomaa[i]   = stOld.dRhomaa[i];   // Apparent Matrix Grain Density
        stNew.dDTmaa[i]    = stOld.dDTmaa[i];    // Apparent Matrix Grain Acoustic transit time

        // Alternate Lithology Curves

        stNew.dNeutron[i]  = stOld.dNeutron[i];  // Neutron Porosity
        stNew.delta[i]     = stOld.delta[i];     // Neutron - Density Porosity

        // Spectral Gamma Ray Lithology Curves

        stNew.dThU[i]      = stOld.dThU[i];      // Th/U
        stNew.dThK[i]      = stOld.dThK[i];      // Th/K

        // Primary Lithology

        stNew.dLITH[i]     = stOld.dLITH[i];     // Primary Lithology

        // Sonic Computed Variables

        stNew.dVc[i]       = stOld.dVc[i];   // Compression Velocity (ft/sec)
        stNew.dVs[i]       = stOld.dVs[i];   // Shear Velocity (ft/sec)
        stNew.dVsf[i]      = stOld.dVsf[i];  // Fast Shear Velocity (ft/sec)
        stNew.dVss[i]      = stOld.dVss[i];  // Slow Shear Velocity (ft/sec)

        stNew.dVAc[i]      = stOld.dVAc[i];  // Average Compression Velocity (ft/sec)
        stNew.dVAs[i]      = stOld.dVAs[i];  // Average Shear Velocity (ft/sec)

        stNew.dTc[i]       = stOld.dTc[i];   // Compression Time (milli-sec)
        stNew.dTs[i]       = stOld.dTs[i];   // Shear Time (milli-sec)
        stNew.dTsf[i]      = stOld.dTsf[i];  // Fast Shear Time (milli-sec)
        stNew.dTss[i]      = stOld.dTss[i];  // Slow Shear Time (milli-sec)

        stNew.dAIc[i]      = stOld.dAIc[i];  // Compression Acoustic Impedence (ft/sec * gm/cc)
        stNew.dAIs[i]      = stOld.dAIs[i];  // Shear Acoustic Impedence (ft/sec * gm/cc)
        stNew.dAIsf[i]     = stOld.dAIsf[i]; // Fast Shear Acoustic Impedence (ft/sec * gm/cc)
        stNew.dAIss[i]     = stOld.dAIss[i]; // Slow Shear Acoustic Impedence (ft/sec * gm/cc)

//        stNew.sTops[i]     = new String(stOld.sTops[i]); // Name assign to bed

        for (int j=0; j<stNew.iTotal; j++)
          stNew.iComp[i][j] = stOld.iComp[i][j];
	  }
	}

    return (stNew);
  }

  /** Method transfer()
   *  <p> This method will transfer the contents of the data structure to the new and
   *      delete the old structure.
   * @param   stOld = Old Data Structure
   * @return  stNew = New Data Structure
   */

  public static eventFileDataStruct transfer(eventFileDataStruct stOld)
  {
    eventFileDataStruct stNew = null;

    if (stOld != null)
    {
	  stNew = copy( stOld );

	  stOld.delete();
	  stOld = null;
	}

    return (stNew);
  }

  /** Method computeMath()
   * <p> This method will compute the lithology curves
   * @param  stC   = Seismic Data Structure
   * @param  fluid = The Fluid Type (Fresh=0, Salt=1, Oil=2)
   * @return stC   = Seismic Data Well Structure
   */

  public static eventFileDataStruct computeMath(eventFileDataStruct st, int fluid)
  {
	int    _DENSITY  = math.mathLAS._DENSITY;
	int    _SONIC    = math.mathLAS._SONIC;
	int    _CALCITE  = math.mathLAS._CALCITE;
	int    _FRESH    = math.mathLAS._FRESH;

	double dNull     = 0.0;
	double dPHI      = 0.0;
	double dTemp     = 0.0;

    double dShaly    = iqstrat.iqstratShaleStruct.SHALY;
    double dShale    = iqstrat.iqstratShaleStruct.SHALE;
    double dHotShale = iqstrat.iqstratShaleStruct.SHALE_HOT;

	if (st != null)
	{
	  dNull = st.dNull;

	  for (int i=0; i<st.iRows; i++)
	  {
		// Make sure that both DTC and SPHI exist.

        if ((st.dRHOB[i] > dNull) && (st.dDPHI[i] == dNull))
        {
		  st.dDPHI[i] = math.mathLAS.computePHI( st.dRHOB[i], _DENSITY, _CALCITE, fluid );
          st.iDPHI    = 1;
		}
		else if ((st.dRHOB[i] == dNull) && (st.dDPHI[i] > dNull))
		{
	      st.dRHOB[i] = math.mathLAS.computeLOG( st.dDPHI[i], _DENSITY,_CALCITE, fluid, dNull );
          st.iRHOB    = 1;
		}

		// Make sure that both RHOB and DPHI exist.

        if ((st.dDTC[i] > dNull) && (st.dSPHI[i] == dNull))
        {
		  st.dSPHI[i] = math.mathLAS.computePHI( st.dDTC[i], _SONIC, _CALCITE, fluid );
          st.iSPHI    = 1;
		}
		else if ((st.dDTC[i] == dNull) && (st.dSPHI[i] > dNull))
		{
	      st.dDTC[i] = math.mathLAS.computeLOG( st.dDPHI[i], _SONIC,_CALCITE, fluid, dNull );
          st.iDTC    = 1;
		}

		dPHI = dNull;
		if ((st.dDPHI[i] > dNull) && (st.dNPHI[i] > dNull))
		{
		  dPHI = math.mathLAS.computeAvgPHI( st.dNPHI[i], st.dDPHI[i] );
		}
		else if (st.dNPHI[i] > dNull)
		{
		  dPHI = st.dNPHI[i];
		}
		else if (st.dDPHI[i] > dNull)
		{
		  dPHI = st.dDPHI[i];
		}
		else if (st.dSPHI[i] > dNull)
		{
		  dPHI = st.dSPHI[i];
		}

        // Compute RHOmaa (Apparent Grain Density)

		if ((st.dRHOB[i] > dNull) && (dPHI > dNull))
		{
		  st.dRhomaa[i] = math.mathLAS.computeRhomaa( fluid, st.dRHOB[i], dPHI );
		  st.iRhomaa    = 1;
		}

        // Compute Umaa (Apparent Photoelectric Factor)

		if ((st.dRHOB[i] > dNull) && (st.dPE[i] > dNull) && (dPHI > dNull))
		{
		  st.dUmaa[i] = math.mathLAS.computeUmaa( fluid, st.dRHOB[i], st.dPE[i], dPHI );
		  st.iUmaa    = 1;
		}
		else if ((st.dPE[i] > dNull) && (dPHI > dNull))
		{
		  st.dUmaa[i] = math.mathLAS.computeUmaa( st.dPE[i], dPHI );
		  st.iUmaa    = 1;
		}

        // Compute DTmaa (Apparent Acoustic Transit Time)

		if ((st.dDTC[i] > dNull) && (dPHI > dNull))
		{
		  st.dDTmaa[i] = math.mathLAS.computeDTMAA( fluid, st.dDTC[i], dPHI );
		  st.iDTmaa    = 1;
		}

        // Compute Porosity Difference (NPHI - DPHI)

		if ((st.dDPHI[i] > dNull) && (st.dNPHI[i] > dNull))
		{
		  st.delta[i]    = st.dNPHI[i] - st.dDPHI[i];
          st.iDelta      = 1; // Neutron - Density Porosity
		  st.dNeutron[i] = st.dNPHI[i];
          st.iNeutron    = 1; // Neutron Porosity
		}

		if ((st.dTh[i] > dNull) && st.dU[i] > dNull)
		{
		  dTemp = st.dU[i];
          if (dTemp == 0.0) dTemp = .0001;

		  st.dThU[i] = st.dTh[i] / dTemp;
		  st.iThU    = 1;
		}

		if ((st.dTh[i] > dNull) && st.dK[i] > dNull)
		{
		  dTemp = st.dK[i];
          if (dTemp == 0.0) dTemp = .0001;

		  st.dThK[i] = st.dTh[i] / dTemp;
		  st.iThK    = 1;
		}

        // Determine a General "Lithology" from Log or Core Data
        // -- Bin the minerals

        if ((st.dGR[i] > dNull) && (dPHI > dNull) &&
            ((st.dRHOB[i] > dNull) || (st.dDTC[i] > dNull)))
        {
          st.dLITH[i] = (double) lith.math.lithMathRules.getLithology(
                          fluid,
                          dShaly, dShale,  dHotShale,
                          st.dGR[i], dPHI, st.dRHOB[i], st.dPE[i], st.dDTC[i],
                          dNull );
          st.iLITH    = 1;
		}
	  }
	}

	return (st);
  }

  /** Method backFill()
   *  <p> This method will back fill the zonation data replacing nulls or 0's
   *      with the first non null or non zero term for each data type.
   * @param  st = Seismic Data Structure
   * @return st = Seismic Data Well Structure
   */

  public static eventFileDataStruct backFill(eventFileDataStruct st)
  {
    int    iColumn[] = { -1, -1, -1, -1, -1, -1, -1, -1,
                         -1, -1, -1, -1, -1, -1, -1, -1, -1 };
    double dVal[]    = { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                         0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 };
	double data      = 0.0;

	if (st != null)
	{
      for (int i=0; i<st.iRows; i++)
      {
        for (int j=3; j<event.eventStandardTools.TOTAL_LAS; j++)
        {
          if (st.checkData(j))
          {
			if (iColumn[j] == -1)
			{
  	          data = st.getRow(j, i);
			  if ((data != st.dNull) && (data != 0.0))
			  {
				dVal[j]    = data;
				iColumn[j] = i;
			  }
		    }
		  }
		}
	  }

      for (int i=0; i<st.iRows; i++)
      {
        for (int j=0; j<event.eventStandardTools.TOTAL_LAS; j++)
        {
          if (st.checkData(j))
          {
            data = st.getRow(j, i);
            if (i < iColumn[j])
              st.setRow(j, i, dVal[j]);
		  }
		}
	  }
	}

	return (st);
  }

  /** Method fillSonicData()
   *  <p> This method will back fill the zonation data replacing nulls or 0's
   *      with the first non null or non zero term for each data type.
   * @param  st = Seismic Data Structure
   * @return st = Seismic Data Well Structure
   */

  public static eventFileDataStruct fillSonicData(eventFileDataStruct st)
  {
	if (st != null)
	{
      for (int i=0; i<st.iRows; i++)
      {
		if ((st.iDTSF > -1) && (st.iDTSS > -1))
        {
		  if ((st.dDTSF[i] > -1) && (st.dDTSS[i] > -1))
		  {
		    st.iDTS    = 1;
		    st.dDTS[i] = (st.dDTSF[i] + st.dDTSS[i]) / 2.0;
		    st.dDTS[i] = (double) ( (int) 1000.0 * st.dDTS[i] ) / 1000.0;
		  }
		}
	  }
	}

	return (st);
  }

  /** Method computeMath()
   * <p> This method will compute the lithology curves
   * @param  stC = Seismic Data Structure
   * @return stC = Seismic Data Well Structure
   */

  public static eventFileDataStruct computeMath(eventFileDataStruct stC)
  {
    int    i        = 0;
    int    idata    = 0;

    if (stC != null)
    {
	  // Compute Sonic Porosity

      if (stC.iDTC > -1)
      {
        if (stC.iSPHI == -1)
	    {
		  stC.dSPHI = math.mathLAS.computePHI( stC.iRows,
                                               stC.dDTC,
                                               math.mathLAS._SONIC,
                                               math.mathLAS._CALCITE,
                                               math.mathLAS._FRESH,
                                               stC.dNull );
          stC.iSPHI = 1;
		}
	  }

      if (stC.iRHOB > -1)
      {
	    if (stC.iDPHI == -1)
	    {
		  stC.dDPHI = math.mathLAS.computePHI( stC.iRows,
                                               stC.dRHOB,
                                               math.mathLAS._DENSITY,
                                               math.mathLAS._CALCITE,
                                               math.mathLAS._FRESH,
                                               stC.dNull );
          stC.iDPHI = 1;
		}
	  }

      // Compute the Rhmaa Umaa Lithology Curves

      if ( (stC.iPE > -1) && (stC.iRHOB > -1) && (stC.iNPHI > -1))
      {
        double dOut[][] = null;

        dOut = math.mathLAS.computeRhomaaUmaa( math.mathLAS._FRESH,
                                               stC.iRows,
                                               stC.dNPHI,
                                               stC.dRHOB,
                                               stC.dPE,
                                               stC.dNull );

        if (dOut != null)
        {
          for (i=0; i<stC.iRows; i++)
          {
            stC.dRhomaa[i] = dOut[0][i]; // Apparent Matrix Grain Density
            stC.dUmaa[i]   = dOut[1][i]; // Apparent Volumetric Photoelectric
          }
        }

        stC.iUmaa = 1; // Apparent Volumetric Photoelectric factor
        stC.iRhomaa = 1; // Apparent Matrix Grain Density
      }
      else if ( (stC.iPE > -1) && (stC.iRHOB > -1) && (stC.iDTC > -1))
      {
        double dOut[][] = null;

	    if (stC.iSPHI == -1)
	    {
		  stC.dSPHI = math.mathLAS.computePHI( stC.iRows,
                                               stC.dDTC,
                                               math.mathLAS._SONIC,
                                               math.mathLAS._CALCITE,
                                               math.mathLAS._FRESH,
                                               stC.dNull );
          stC.iSPHI = 1;
		}

        dOut = math.mathLAS.computeRhomaaUmaa( math.mathLAS._FRESH,
                                               stC.iRows,
                                               stC.dSPHI,
                                               stC.dRHOB,
                                               stC.dPE,
                                               stC.dNull );

        if (dOut != null)
        {
          for (i=0; i<stC.iRows; i++)
          {
            stC.dRhomaa[i] = dOut[0][i]; // Apparent Matrix Grain Density
            stC.dUmaa[i]   = dOut[1][i]; // Apparent Volumetric Photoelectric
          }
        }

        stC.iUmaa = 1; // Apparent Volumetric Photoelectric factor
        stC.iRhomaa = 1; // Apparent Matrix Grain Density
      }
      else if ((stC.iRHOB > -1) && (stC.iNPHI > -1))
      {
        stC.dRhomaa = math.mathLAS.computeRhomaa( math.mathLAS._FRESH,
                                                  stC.iRows,
                                                  stC.dNPHI,
                                                  stC.dRHOB,
                                                  stC.dNull );

        stC.iRhomaa = 1; // Apparent Matrix Grain Density
      }
      else if ((stC.iRHOB > -1) && (stC.iDTC > -1))
      {
	    if (stC.iSPHI == -1)
	    {
		  stC.dSPHI = math.mathLAS.computePHI( stC.iRows,
                                               stC.dDTC,
                                               math.mathLAS._SONIC,
                                               math.mathLAS._CALCITE,
                                               math.mathLAS._FRESH,
                                               stC.dNull );
          stC.iSPHI = 1;
		}

        stC.dRhomaa = math.mathLAS.computeRhomaa( math.mathLAS._FRESH,
                                                  stC.iRows,
                                                  stC.dSPHI,
                                                  stC.dRHOB,
                                                  stC.dNull );

        stC.iRhomaa = 1; // Apparent Matrix Grain Density
      }
      else if (stC.iRHOB > -1)
      {
	    if (stC.iDPHI == -1)
	    {
		  stC.dDPHI = math.mathLAS.computePHI( stC.iRows,
                                               stC.dDTC,
                                               math.mathLAS._SONIC,
                                               math.mathLAS._CALCITE,
                                               math.mathLAS._FRESH,
                                               stC.dNull );
          stC.iDPHI = 1;
		}

        stC.dRhomaa = math.mathLAS.computeRhomaa( math.mathLAS._FRESH,
                                                  stC.iRows,
                                                  stC.dDPHI,
                                                  stC.dRHOB,
                                                  stC.dNull );

        stC.iRhomaa = 1; // Apparent Matrix Grain Density
	  }

      if ((stC.iRHOB > -1) && (stC.iNPHI > -1) && (stC.iDTC > -1))
      {
        stC.dDTmaa = math.mathLAS.computeDTmaa( math.mathLAS._FRESH,
                                                stC.iRows,
                                                stC.dNPHI,
                                                stC.dRHOB,
                                                stC.dDTC,
                                                stC.dNull );

        stC.iDTmaa = 1; // Apparent Matrix Grain Acoustic transit time
      }
      else if ((stC.iRHOB > -1) && (stC.iDTC > -1))
      {
	    if (stC.iSPHI == -1)
	    {
		  stC.dSPHI = math.mathLAS.computePHI( stC.iRows,
                                               stC.dDTC,
                                               math.mathLAS._SONIC,
                                               math.mathLAS._CALCITE,
                                               math.mathLAS._FRESH,
                                               stC.dNull );
          stC.iSPHI = 1;
		}

        stC.dDTmaa = math.mathLAS.computeDTmaa( math.mathLAS._FRESH,
                                                stC.iRows,
                                                stC.dSPHI,
                                                stC.dRHOB,
                                                stC.dDTC,
                                                stC.dNull );

        stC.iDTmaa = 1; // Apparent Matrix Grain Acoustic transit time
	  }
      else if (stC.iDTC > -1)
      {
	    if (stC.iSPHI == -1)
	    {
		  stC.dSPHI = math.mathLAS.computePHI( stC.iRows,
                                               stC.dDTC,
                                               math.mathLAS._SONIC,
                                               math.mathLAS._CALCITE,
                                               math.mathLAS._FRESH,
                                               stC.dNull );
          stC.iSPHI = 1;
		}

        stC.dDTmaa = math.mathLAS.computeDTmaa( math.mathLAS._FRESH,
                                                stC.iRows,
                                                stC.dSPHI,
                                                stC.dDTC,
                                                stC.dNull );

        stC.iDTmaa = 1; // Apparent Matrix Grain Acoustic transit time
	  }

      // Compute the Porosity Difference Lithology Curves

      if ( (stC.iRHOB > -1) && (stC.iNPHI > -1))
      {
        double dOut[][] = null;

        dOut = math.mathLAS.computePorosityDifference( math.mathLAS._FRESH,
                                                       stC.iRows,
                                                       stC.dNPHI,
                                                       stC.dRHOB,
                                                       stC.dNull );

        if (dOut != null)
        {
          for (i=0; i<stC.iRows; i++)
          {
            stC.dNeutron[i] = dOut[0][i]; // Neutron Porosity
            stC.delta[i]    = dOut[1][i]; // Neutron - Density Porosity
          }
        }

        stC.iNeutron = 1; // Neutron Porosity
        stC.iDelta = 1; // Neutron - Density Porosity
      }

      // Compute the Spectral Gamma Ray Lithology Curves

      if ( (stC.iTh > -1) && (stC.iU > -1) && (stC.iK > -1))
      {
        double dOut[][] = null;

        dOut = math.mathLAS.computeSpectralGR( stC.iRows,
                                               stC.dTh,
                                               stC.dU,
                                               stC.dK,
                                               stC.dNull );

        if (dOut != null)
        {
          for (i=0; i<stC.iRows; i++)
          {
            stC.dThU[i] = dOut[0][i]; // Thorium-Uranium Ratio
            stC.dThK[i] = dOut[1][i]; // Thorium-Potassium Ratio
          }
        }

        stC.iThU = 1; // Th/U
        stC.iThK = 1; // Th/K
      }
    }
     return (stC);
  }

  /** Method computeSeismicMath()
   * <p> This method will compute the lithology curves
   * @param  st = Seismic Data Structure
   * @return st = Seismic Data Structure
   */

  public static eventFileDataStruct computeSeismicMath(eventFileDataStruct st)
  {
	double dLimits[] = null;

	if (st != null)
	{
      dLimits  = event.math.eventMath.getDepthRange(
		           st.iRows, st.dTop, st.dBase, st.dNull );
      st.dStart = dLimits[0];
      if (st.dStart > 0.0)
        st.dStart = 0.0;
      st.dEnd   = dLimits[1];

      if ((st.iDTS == -1) && (st.iDTSF > -1) && (st.iDTSS > -1))
      {
		st.dDTS = event.math.eventMath.getAvgDTs( st.iRows, st.dDTSS, st.dDTSF);
        st.iDTS = 1;
	  }

      if ((st.iDTC > -1) && (st.iDTS == -1) && (st.iLITH > -1))
      {
		st.dDTS = event.math.eventMath.getDTs( st.iRows, st.dDTC, st.dLITH);
        st.iDTS = 1;
	  }

      if (st.iDTC > -1)
      {
        st.dTc = event.math.eventMath.getTime(
			       st.iRows, st.dTop, st.dDTC, st.dNull );
        st.iTc = 1;
	  }

      if (st.iDTS > -1)
      {
        st.dTs = event.math.eventMath.getTime(
			       st.iRows, st.dTop, st.dDTS, st.dNull );
        st.iTs = 1;
      }

      if (st.iDTSF > -1)
      {
        st.dTsf = event.math.eventMath.getTime(
			        st.iRows, st.dTop, st.dDTSF, st.dNull );
        st.iTsf = 1;
      }

      if (st.iDTSS > -1)
      {
        st.dTss = event.math.eventMath.getTime(
			        st.iRows, st.dTop, st.dDTSS, st.dNull );
        st.iTss = 1;
      }

      if (st.iDTC > -1)
      {
        st.dVc = event.math.eventMath.getVelocity( st.iRows, st.dDTC, st.dNull );
        st.iVc = 1;
      }

      if (st.iDTS > -1)
      {
        st.dVs = event.math.eventMath.getVelocity( st.iRows, st.dDTS, st.dNull );
        st.iVs = 1;
      }

      if (st.iDTSF > -1)
      {
        st.dVsf = event.math.eventMath.getVelocity( st.iRows, st.dDTSF, st.dNull );
        st.iVsf = 1;
      }

      if (st.iDTSS > -1)
      {
        st.dVss = event.math.eventMath.getVelocity( st.iRows, st.dDTSS, st.dNull );
        st.iVss = 1;
      }

      if ((st.iVc > -1) && (st.iRHOB > -1))
      {
        st.dAIc = event.math.eventMath.getAI(
			        st.iRows, st.dVc, st.dRHOB, st.dNull );
        st.iAIc = 1;
      }

      if ((st.iVs > -1) && (st.iRHOB > -1))
      {
        st.dAIs = event.math.eventMath.getAI(
			        st.iRows, st.dVs, st.dRHOB, st.dNull );
        st.iAIs = 1;
      }

      if ((st.iVsf > -1) && (st.iRHOB > -1))
      {
        st.dAIsf = event.math.eventMath.getAI(
			         st.iRows, st.dVsf, st.dRHOB, st.dNull );
        st.iAIsf = 1;
      }

      if ((st.iVss > -1) && (st.iRHOB > -1))
      {
        st.dAIss = event.math.eventMath.getAI(
			         st.iRows, st.dVss, st.dRHOB, st.dNull );
        st.iAIss = 1;
      }

      if ((st.iTc > -1) && (st.iTs > -1))
      {
        st.dDTSP = event.math.eventMath.getDeltaT(
			         st.iRows, st.dTc, st.dTs, st.dNull );
        st.iDTSP = 1;
      }

      if ((st.iTc > -1) && (st.iTsf > -1))
      {
        st.dDTSPf = event.math.eventMath.getDeltaT(
			          st.iRows, st.dTc, st.dTsf, st.dNull );
        st.iDTSPf = 1;
      }

      if ((st.iTc > -1) && (st.iTss > -1))
      {
        st.dDTSPs = event.math.eventMath.getDeltaT(
			          st.iRows, st.dTc, st.dTss, st.dNull );
        st.iDTSPs = 1;
      }

      if (st.iVc > -1)
      {
        st.dVAc = event.math.eventMath.getAvgVelocity(
			        st.iRows, st.dTop, st.dBase, st.dVc, st.dNull );
	    st.iVAc = 1;
	  }

      if (st.iVs > -1)
      {
        st.dVAs = event.math.eventMath.getAvgVelocity(
			        st.iRows, st.dTop, st.dBase, st.dVs, st.dNull );
	    st.iVAs = 1;
	  }
	}

	return (st);
  }

  /** Method mapTops()
   *  <p> This method will compute the Lithology Composition.
   * @param  st        = Seismic Data Structure
   * @param  stTops    = Stratigraphic List Structure
   * @return st        = Seismic Data Structure
   */

  public static eventFileDataStruct mapTops( eventFileDataStruct st,
                                             stratListStruct stTops)
  {
	double depth1 = 0.0;
	double depth2 = 0.0;

	if ((st != null) && (stTops != null))
	{
	  for (int i=0; i<st.iRows; i++)
	  {
		depth1 = st.dTop[i];
		depth2 = st.dBase[i];

		for (int j=0; j<stTops.iCount; j++)
		{
		  if ((stTops.stItem[j].depthStart >= depth1) &&
		      (stTops.stItem[j].depthStart < depth2))
		  {
			st.sName[i] = new String(stTops.stItem[j].sName);
//			st.sTops[i] = new String(stTops.stItem[j].sName);
		  }
		}
	  }
	}

	return (st);
  }

  /** Method computeLithology()
   *  <p> This method will compute the Lithology Composition.
   * @param  st        = Seismic Data Structure
   * @param  stSymbols = Lithology Symbols Data List Structure
   * @param  fluid     = The Fluid Type (Fresh=0, Salt=1, Oil=2)
   * @return st        = Seismic Data Structure
   */

  public static eventFileDataStruct computeLithology(
	              eventFileDataStruct st,
                  lithologySymbolsListStruct stSymbols,
                  int fluid )
  {
	int    iLith[] = null;

	if (st != null)
	{
	  iLith = new int[st.iTotal];

	  for (int i=0; i<st.iRows; i++)
	  {
		st.iComp[i] = computeLithology( st.dGR[i],   // Gamma Ray
                                        st.dRHOB[i], // Bulk Density
                                        st.dNPHI[i], // Neutron Porosity
                                        st.dPE[i],   // Photoelectric
                                        st.dDTC[i],  // Acousti Transit Time
                                        st.dNull,    // LAS File Null Value
                                        st.dLITH[i], // Primary Lithology ID
                                        st.iTotal,   // Total Number of Cells
                                        stSymbols,   // Lithology Symbols Data
                                        fluid );     // Type of Fluid ID

        for (int j=0; j<st.iTotal; j++)
        {
		  st.iComp[i][j] = iLith[j];
		}
	  }
	}

	return (st);
  }

  /** Method computeLithology()
   *  <p> This method will compute the Lithology Composition.
   * @param  iRows     = The total number of rows
   * @param  dGR       = Array of Gamma Ray Log Values
   * @param  dRHOB     = Array of Bulk Density Log Values
   * @param  dNPHI     = Array of Neutron Porosity Log Values
   * @param  dPE       = Array of Photoelectric Log Values
   * @param  dDT       = Array of Acoustic Transit Time
   * @param  dNull     = LAS File Null Value
   * @param  dLITH     = Array of Primary Lithology Indicator
   * @param  iTotal    = Total Number of Composition Cells
   * @param  stSymbols = Lithology Symbols Data List Structure
   * @param  fluid     = The Fluid Type (Fresh=0, Salt=1, Oil=2)
   * @return st    = Seismic Data Structure
   */

  public static int[] computeLithology( double dGR,
                                        double dRHOB,
                                        double dNPHI,
                                        double dPE,
                                        double dDT,
                                        double dNull,
                                        double dLithology,
                                        int    iTotal,
                                        lithologySymbolsListStruct stSymbols,
                                        int fluid)
  {
	int    _DENSITY  = math.mathLAS._DENSITY;
	int    _SONIC    = math.mathLAS._SONIC;
	int    _CALCITE  = math.mathLAS._CALCITE;

	int    iLith[]     = new int[iTotal];

    int i, j, k, m, n, p;
    int    iTemp       = 0;
    double dSum        = 0.0;
    double dTemp       = 0.0;
    int    iDolomite   = 0;
    int    iLimestone  = 0;
    int    iSand       = 0;
    int    iCount      = 0;
    int    iStart      = 0;
    int    iEnd        = 0;

    int    iLithology  = 0;
    int    iLithColor  = 0;
    int    iContinue   = 0;

    int    len         = 0;
    int    iMin[]      = null;
    int    iMin_Lith[] = null;
    int    iMin_Col[]  = null;
    double dLith[]     = { 0.0, 0.0, 0.0, 0.0 , 0.0 };
    double dIN[]       = null;
    double dMat[][]    = null;
    double dPHI        = dNull;
    double dDPHI       = dNull;
    double dSPHI       = dNull;

    final int _RU = 0;   // RHOmaa-Umaa Data Present
    final int _PD = 1;   // Porosity Difference - NPHI
    final int _RT = 2;   // RHOmaa-DTmaa Data Present
    int    iData = -1;

    if ((dGR > dNull) && (dRHOB > dNull) && (dNPHI > dNull) && (dPE > dNull))
      iData = _RU;
    else if ((dGR > dNull) && (dRHOB> dNull) && (dNPHI > dNull))
      iData = _PD;
    else if ((dGR > dNull) && (dRHOB> dNull) && (dDT > dNull))
      iData = _RT;

    for (j=0; j<iTotal; j++)
    {
      iLith[j] = (int) dLithology;
    }

    if (iData > -1)
    {
	  // Load default Lithology into every cell

      iLithology = (int) dLithology;

      // Compute Density Porosity from Bulk Density

	  if (dRHOB > dNull)
        dDPHI = math.mathLAS.computePHI( dRHOB, _DENSITY, _CALCITE, fluid );

      // Compute Sonic Porosity from Acoustic Transit Time

	  if (dDT > dNull)
	   dSPHI = math.mathLAS.computePHI( dDT, _SONIC, _CALCITE, fluid );

      // Compute Total Porosity

	  dPHI = dNull;
	  if ((dDPHI > dNull) && (dNPHI > dNull))
	  {
	    dPHI = math.mathLAS.computeAvgPHI( dNPHI, dDPHI );
	  }
	  else if (dNPHI > dNull)
	  {
	    dPHI = dNPHI;
	  }
	  else if (dDPHI > dNull)
	  {
	    dPHI = dDPHI;
	  }
	  else if (dSPHI > dNull)
	  {
	    dPHI = dSPHI;
	  }

      // Get Composition Matrices

      switch (iData)
      {
        /* ------------------------------------------------------- *
         * ----------------- RHOMAA - UMAA LOGS ------------------ *
         * ------------------------------------------------------- */
		case _RU:
		  switch (iLithology)
		  {
            // ....................................................
            // -- GENERAL SEDIMENTARY LITHOLOGY
		    // QUARTZ - CALCITE - DOLOMITE TRIANGLE
            // ....................................................
            case lith.lithology.lithologySymbolsStruct._DOLOMITE:
            case lith.lithology.lithologySymbolsStruct._LIMESTONE:
            case lith.lithology.lithologySymbolsStruct._SANDSTONE:
            case lith.lithology.lithologySymbolsStruct._SHALY_DOLO:
            case lith.lithology.lithologySymbolsStruct._SHALY_CARB:
            case lith.lithology.lithologySymbolsStruct._SHALY_SAND:
              iMin      = lith.math.lithMath._RU;
              iMin_Lith = lith.math.lithMath._RU_LITH;
              iMin_Col  = lith.math.lithMath._RU_COL;

              dIN = lith.math.lithMath.getRU( lith.math.lithMath._CALCITE,
                                              fluid, dRHOB, dPE, dPHI );

              len = iMin.length;
              iContinue = 1;
              break;
            // ....................................................
            // -- ARKOSE LITHOLOGY
		    // K-FELDSPAR - QUARTZ - CALCITE - ILLITE QUADRANGLE
            // ....................................................
            case lith.lithology.lithologySymbolsStruct._ARKOSE:
              iMin      = lith.math.lithMath._ARKOSE;
              iMin_Lith = lith.math.lithMath._ARKOSE_LITH;
              iMin_Col  = lith.math.lithMath._RUGR_COL;

              dIN = lith.math.lithMath.getRUGR( lith.math.lithMath._CALCITE,
                                                fluid, dRHOB, dPE, dPHI, dGR );

              len = iMin.length;
              iContinue = 1;
              break;
            // ....................................................
            // -- IGNEOUS ROCKS - GRANITE LITHOLOGY
		    // MUSCOVITE - Na-PLAGIOCLASE - QUARTZ, K-FELDSPAR QUADRANGLE
            // ....................................................
            case lith.lithology.lithologySymbolsStruct._GRANITE:
              iMin      = lith.math.lithMath._GRANITE;
              iMin_Lith = lith.math.lithMath._GRANITE_LITH;
              iMin_Col  = lith.math.lithMath._RUGR_COL;

              dIN = lith.math.lithMath.getRUGR( lith.math.lithMath._CALCITE,
                                                fluid, dRHOB, dPE, dPHI, dGR );

              len = iMin.length;
              iContinue = 1;
              break;
            // ....................................................
            // -- GENERAL IGNEOUS ROCKS LITHOLOGY
		    // ULTRA MAFIC - Ca-PLAGIOCLASE - QUARTZ, K-FELDSPAR QUADRANGLE
            // ....................................................
            case lith.lithology.lithologySymbolsStruct._IGNEOUS:
              iMin      = lith.math.lithMath._IGNEOUS;
              iMin_Lith = lith.math.lithMath._IGNEOUS_LITH;
              iMin_Col  = lith.math.lithMath._RUGR_COL;

              dIN = lith.math.lithMath.getRUGR( lith.math.lithMath._CALCITE,
                                                fluid, dRHOB, dPE, dPHI, dGR );

              len = iMin.length;
              iContinue = 1;
              break;
            // ....................................................
            // -- ANHYDRITE LITHOLOGY
		    // DOLOMITE - LIMESTONE - ANHYDRITE TRIANGLE
            // ....................................................
            case lith.lithology.lithologySymbolsStruct._ANHYDRITE:
              iMin      = lith.math.lithMath._ANHY;
              iMin_Lith = lith.math.lithMath._ANHY_LITH;
              iMin_Col  = lith.math.lithMath._RU_COL;

              dIN = lith.math.lithMath.getRU( lith.math.lithMath._CALCITE,
                                              fluid, dRHOB, dPE, dPHI );

              len = iMin.length;
              iContinue = 1;
              break;
            // ....................................................
            // -- GYPSUM LITHOLOGY
  		    // DOLOMITE - LIMESTONE - GYPSUM TRIANGLE
            // ....................................................
            case lith.lithology.lithologySymbolsStruct._GYPSUM:
              iMin      = lith.math.lithMath._GYP;
              iMin_Lith = lith.math.lithMath._GYP_LITH;
              iMin_Col  = lith.math.lithMath._RU_COL;

              dIN = lith.math.lithMath.getRU( lith.math.lithMath._CALCITE,
                                              fluid, dRHOB, dPE, dPHI );

              len = iMin.length;
              iContinue = 1;
              break;
            // ....................................................
            // -- SALT & COAL DEFAULT LITHOLOGIES
            // ....................................................
            case lith.lithology.lithologySymbolsStruct._HALITE:
            case lith.lithology.lithologySymbolsStruct._COAL:
              break;
            // ....................................................
            // -- SHALE/CLAY LITHOLOGY
			// SMECTITE - ILLITE - CALCITE - QUARTZ QUADRANGLE
            // ....................................................
            case lith.lithology.lithologySymbolsStruct._SHALE:
              iMin      = lith.math.lithMath._RUGR;
              iMin_Lith = lith.math.lithMath._RUGR_LITH;
              iMin_Col  = lith.math.lithMath._RUGR_COL;

              dIN = lith.math.lithMath.getRUGR( lith.math.lithMath._CALCITE,
                                                fluid, dRHOB, dPE, dPHI, dGR );

              len = iMin.length;
              iContinue = 1;
              break;
	      }
		  break;

        /* ------------------------------------------------------- *
         * ---- NEUTRON POROSITY & POROSITY DIFFERENCE LOGS ------ *
         * ------------------------------------------------------- */
		case _PD:
          switch (iLithology)
          {
            // ....................................................
            // -- GENERAL SEDIMENTARY LITHOLOGY
		    // QUARTZ - CALCITE - DOLOMITE TRIANGLE
            // ....................................................
            case lith.lithology.lithologySymbolsStruct._DOLOMITE:
            case lith.lithology.lithologySymbolsStruct._LIMESTONE:
            case lith.lithology.lithologySymbolsStruct._SANDSTONE:
            case lith.lithology.lithologySymbolsStruct._SHALY_DOLO:
            case lith.lithology.lithologySymbolsStruct._SHALY_CARB:
            case lith.lithology.lithologySymbolsStruct._SHALY_SAND:
              iMin      = lith.math.lithMath._RU;
              iMin_Lith = lith.math.lithMath._RU_LITH;
              iMin_Col  = lith.math.lithMath._PD_COL;

              dIN = lith.math.lithMath.getPD( lith.math.lithMath._CALCITE,
                                              fluid, dRHOB, dPHI );

              len       = iMin.length;
              iContinue = 1;
              break;
            // ....................................................
            // -- ARKOSE LITHOLOGY
            // K-FELDSPAR - QUARTZ - CALCITE - ILLITE QUADRANGLE
            // ....................................................
            case lith.lithology.lithologySymbolsStruct._ARKOSE:
              iMin      = lith.math.lithMath._ARKOSE;
              iMin_Lith = lith.math.lithMath._ARKOSE_LITH;
              iMin_Col  = lith.math.lithMath._PDGR_COL;

              dIN = lith.math.lithMath.getPDGR( lith.math.lithMath._CALCITE,
                                                fluid, dRHOB, dPHI, dGR );

              len = iMin.length;
              iContinue = 1;
              break;
            // ....................................................
            // -- IRONSTONE DEFAULT LITHOLOGIES
            // ....................................................
            case lith.lithology.lithologySymbolsStruct._IRONSTONE:
              break;
            // ....................................................
            // -- ANHYDRITE LITHOLOGY
      	    // DOLOMITE - LIMESTONE - ANHYDRITE TRIANGLE
            // ....................................................
            case lith.lithology.lithologySymbolsStruct._ANHYDRITE:
              iMin      = lith.math.lithMath._ANHY;
              iMin_Lith = lith.math.lithMath._ANHY_LITH;
              iMin_Col  = lith.math.lithMath._PD_COL;

              dIN = lith.math.lithMath.getPD( lith.math.lithMath._CALCITE,
                                              fluid, dRHOB, dPHI );

              len = iMin.length;
              iContinue = 1;
              break;
            // ....................................................
            // -- GYPSUM LITHOLOGY
			// DOLOMITE - LIMESTONE - GYPSUM TRIANGLE
            // ....................................................
            case lith.lithology.lithologySymbolsStruct._GYPSUM:
              iMin      = lith.math.lithMath._GYP;
              iMin_Lith = lith.math.lithMath._GYP_LITH;
              iMin_Col  = lith.math.lithMath._PD_COL;

              dIN = lith.math.lithMath.getPD( lith.math.lithMath._CALCITE,
                                              fluid, dRHOB, dPHI );

              len       = iMin.length;
              iContinue = 1;
              break;
            // ....................................................
            // -- SALT & COAL DEFAULT LITHOLOGIES
            // ....................................................
            case lith.lithology.lithologySymbolsStruct._HALITE:
            case lith.lithology.lithologySymbolsStruct._COAL:
              break;
            // ....................................................
            // -- SHALE/CLAY LITHOLOGY
			// SMECTITE - ILLITE - CALCITE - QUARTZ QUADRANGLE
            // ....................................................
            case lith.lithology.lithologySymbolsStruct._SHALE:
              iMin      = lith.math.lithMath._RUGR;
              iMin_Lith = lith.math.lithMath._RUGR_LITH;
              iMin_Col  = lith.math.lithMath._PDGR_COL;

              dIN = lith.math.lithMath.getPDGR( lith.math.lithMath._CALCITE,
                                                fluid, dRHOB, dPHI, dGR );

              len = iMin.length;
              iContinue = 1;
              break;
          }
		  break;

        /* ------------------------------------------------------- *
         * ------------------ RHOMAA - DT LOGS ------------------- *
         * ------------------------------------------------------- */
		case _RT:
          switch (iLithology)
          {
            // ....................................................
            // -- GENERAL SEDIMENTARY LITHOLOGY
		    // QUARTZ - CALCITE - DOLOMITE TRIANGLE
            // ....................................................
            case lith.lithology.lithologySymbolsStruct._DOLOMITE:
            case lith.lithology.lithologySymbolsStruct._LIMESTONE:
            case lith.lithology.lithologySymbolsStruct._SANDSTONE:
            case lith.lithology.lithologySymbolsStruct._SHALY_DOLO:
            case lith.lithology.lithologySymbolsStruct._SHALY_CARB:
            case lith.lithology.lithologySymbolsStruct._SHALY_SAND:
              iMin      = lith.math.lithMath._RU;
              iMin_Lith = lith.math.lithMath._RU_LITH;
              iMin_Col  = lith.math.lithMath._RT_COL;

              if (dPHI > dNull)
                dIN = lith.math.lithMath.getRT( lith.math.lithMath._CALCITE,
                                                fluid, dRHOB, dDT, dPHI );
              else
                dIN = lith.math.lithMath.getRT( lith.math.lithMath._CALCITE,
                                                fluid, dRHOB, dDT );

              len       = iMin.length;
              iContinue = 1;
              break;
            // ....................................................
            // -- ARKOSE LITHOLOGY
		    // K-FELDSPAR - QUARTZ - CALCITE - ILLITE QUADRANGLE
            // ....................................................
            case lith.lithology.lithologySymbolsStruct._ARKOSE:
              break;
            // ....................................................
            // -- IGNEOUS ROCKS - GRANITE LITHOLOGY
		    // MUSCOVITE - Na-PLAGIOCLASE - QUARTZ, K-FELDSPAR QUADRANGLE
            // ....................................................
            case lith.lithology.lithologySymbolsStruct._GRANITE:
              break;
            // ....................................................
            // -- GENERAL IGNEOUS ROCKS LITHOLOGY
			// ULTRA MAFIC - Ca-PLAGIOCLASE - QUARTZ, K-FELDSPAR QUADRANGLE
            // ....................................................
            case lith.lithology.lithologySymbolsStruct._IGNEOUS:
              iMin      = lith.math.lithMath._IGNEOUS;
              iMin_Lith = lith.math.lithMath._IGNEOUS_LITH;
              iMin_Col  = lith.math.lithMath._RTGR_COL;

              if (dPHI > dNull)
                dIN = lith.math.lithMath.getRTGR( lith.math.lithMath._CALCITE,
                                                  fluid, dRHOB, dDT, dPHI, dGR );
              else
                dIN = lith.math.lithMath.getRTGR( lith.math.lithMath._CALCITE,
                                                  fluid, dRHOB, dDT, dGR );

              len = iMin.length;
              iContinue = 1;
              break;
            // ....................................................
            // -- ANHYDRITE LITHOLOGY
			// DOLOMITE - LIMESTONE - ANHYDRITE TRIANGLE
            // ....................................................
            case lith.lithology.lithologySymbolsStruct._ANHYDRITE:
              iMin      = lith.math.lithMath._ANHY;
              iMin_Lith = lith.math.lithMath._ANHY_LITH;
              iMin_Col  = lith.math.lithMath._RT_COL;

              if (dPHI > dNull)
                dIN = lith.math.lithMath.getRT( lith.math.lithMath._CALCITE,
                                                fluid, dRHOB, dDT, dPHI );
              else
                dIN = lith.math.lithMath.getRT( lith.math.lithMath._CALCITE,
                                                fluid, dRHOB, dDT );

              len       = iMin.length;
              iContinue = 1;
              break;
            // ....................................................
            // -- GYPSUM LITHOLOGY
			// DOLOMITE - LIMESTONE - GYPSUM TRIANGLE
            // ....................................................
            case lith.lithology.lithologySymbolsStruct._GYPSUM:
              break;
            // ....................................................
            // -- SALT & COAL DEFAULT LITHOLOGIES
            // ....................................................
            case lith.lithology.lithologySymbolsStruct._HALITE:
            case lith.lithology.lithologySymbolsStruct._COAL:
              break;
            // ....................................................
            // -- SHALE/CLAY LITHOLOGY
			// SMECTITE - ILLITE - CALCITE - QUARTZ QUADRANGLE
            // ....................................................
            case lith.lithology.lithologySymbolsStruct._SHALE:
              iMin      = lith.math.lithMath._RUGR;
              iMin_Lith = lith.math.lithMath._RUGR_LITH;
              iMin_Col  = lith.math.lithMath._RTGR_COL;

              if (dPHI > dNull)
                dIN = lith.math.lithMath.getRTGR( lith.math.lithMath._CALCITE,
                                                  fluid, dRHOB, dDT, dPHI, dGR );
              else
                dIN = lith.math.lithMath.getRTGR( lith.math.lithMath._CALCITE,
                                                  fluid, dRHOB, dDT, dGR );

              len = iMin.length;
              iContinue = 1;
              break;
          }
		  break;
	  }

      if (iContinue == 1)
	  {
	    // Build Lithology Matrix

        dMat  = lith.math.lithMath.getMatrix(iMin, iMin_Col);

        // Compute Mineral Composition

        dLith = lith.math.lithMath.composition( dIN, dMat );

        // Compute the Total Sum of all Mineral Composition

        dSum = 0.0;
        for (j=0; j<len; j++)
        {
          if (dLith[j] < 0.0)
            dLith[j] = 0.0;

          dSum = dSum + dLith[j];
        }

        // FOR EACH Lithology

        for (j=0; j<len; j++)
        {
          // IF First Lithology set to first 10%.

          if (j==0) iStart = 0;

          // Normalize Lithology to the Sum of all Mineral Composition

          dTemp = (10.0 * dLith[j]) / dSum;
          iTemp = (int) dTemp;
          dTemp = dTemp - (double) iTemp;

          // Compute the End 10% value for the lithology

          iEnd = iStart + (int) ((10.0 * dLith[j]) / dSum);

          // Round Cell up to next 10%

          if (dTemp > 0.5)
            iEnd++;

          // IF total cell count is greater than 100% Set to 100%

          if (iEnd > 10)
            iEnd = 10;

          // IF end cell of the last lithology is only 90% set to 100%

          if (j == len-1)
            if (iEnd == 9)
              iEnd = 10;

          // Retrieve the Mineral Symbols for each lithology

          iLithColor = iMin_Lith[j];

          // If Rock is Shaly or Shale reset Lithology Symbol for
          // General Sedimentary Rock Cells to Shaly

          switch (iLithColor)
          {
            case lith.lithology.lithologySymbolsStruct._DOLOMITE:
              switch (iLithology)
              {
                case lith.lithology.lithologySymbolsStruct._SHALY_CARB:
                case lith.lithology.lithologySymbolsStruct._SHALY_SAND:
                case lith.lithology.lithologySymbolsStruct._SHALE:
                  iLithColor = lith.lithology.lithologySymbolsStruct._SHALY_DOLO;
                  break;
              }
              break;
            case lith.lithology.lithologySymbolsStruct._LIMESTONE:
              switch (iLithology)
              {
                case lith.lithology.lithologySymbolsStruct._SHALY_CARB:
                case lith.lithology.lithologySymbolsStruct._SHALY_SAND:
                case lith.lithology.lithologySymbolsStruct._SHALE:
                  iLithColor = lith.lithology.lithologySymbolsStruct._SHALY_CARB;
                  break;
              }
              break;
            case lith.lithology.lithologySymbolsStruct._SANDSTONE:
              switch (iLithology)
              {
                case lith.lithology.lithologySymbolsStruct._SHALY_CARB:
                case lith.lithology.lithologySymbolsStruct._SHALY_SAND:
                case lith.lithology.lithologySymbolsStruct._SHALE:
                  iLithColor = lith.lithology.lithologySymbolsStruct._SHALY_SAND;
                  break;
              }
              break;
          }

          // Load Lithology Symbol into each cell within the Start and
          // End Cells for lithology

          for (k=iStart; k<iEnd; k++)
          {
            iLith[k] = iLithColor;
          }

          // Start Next Lithology in Array

          iStart = iEnd;
		}
      }
	}

    return (iLith);
  }

  /** Method print()
   *  <p> print the contents of the data structure.
   */

  public static void print(eventFileDataStruct st)
  {
	double data = 0.0;

	if (st != null)
	{
      for (int i=0; i<st.iRows; i++)
      {
		System.out.print(i+" ");
        for (int j=0; j<event.eventStandardTools.TOTAL_LAS; j++)
        {
          if (st.checkData(j))
          {
	        data = st.getRow(j, i);
	        System.out.print(event.eventStandardTools.CURVES[j][1]+"="+data+" ");
		  }
		}
//		System.out.println(st.sTops[i]);
		System.out.println(st.sName[i]);
	  }
	}
  }
}

/*
 *  @version 1.1 02/19/2015
 *  @author  John Victorine
 */
