/*
 * @envTable.java Version 1.1 11/15/2011
 *
 * Copyright (c) 2011 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package horizon.env.gui;

import java.awt.*;
import java.lang.*;
import javax.swing.*;
import javax.swing.table.*;

import gui.guiTable;

import horizon.env.envListStruct;
import horizon.env.envStruct;

/** Class envTable
 *  <p> This Class will display the Formation Tops.
 *
 *  @version 1.1 11/15/2011
 *  @author  John R. Victorine
 */

public class envTable
{
  private int      iRows     = 0;
  private int      iColumns  = 7;
  private String   sColumn[] = {"Start", "End", "Environment", "Type", "Group", "Subgroup", "Description"};
  private int      iColLength[] = { 8,  // Starting Depth
                                    8,  // Ending Depth
                                   20,  // Environment
                                   18,  // Type
                                   18,  // Group
                                   18,  // Subgroup
                                   30 };// Description

  private Object   oData[][] = null;
  private guiTable pTable    = null;

  private envListStruct st      = null;
  private int           iChange = 0;

  /** Constructor envTable()
   *  <p> This is the Constructor for this class.
   */

  public envTable()
  {
    pTable = new guiTable(0, iColumns, sColumn);

    resetColumnWidths();
    pTable.showHorScroll(true);
  }

  /** Constructor envTable()
   *  <p> This is the Constructor for this class.
   * @param st     = The Regions List Data Structure
   */

  public envTable(envListStruct st)
  {
    this.st = st;

    populateList();

    if (iRows > 0)
      pTable = new guiTable(0, iRows, iColumns, sColumn, oData);
    else
      pTable = new guiTable(0, iColumns, sColumn);

    resetColumnWidths();
    pTable.showHorScroll(true);

    if (iChange == 0)
      iRows = 0;

    if (iRows > 0)
      pTable.setSelectedRow(0);
  }

  /** Method close()
   * <p> This method will close the this tabe
   */

  public void close()
  {
    sColumn    = null;
    iColLength = null;
    oData      = null;
    pTable     = null;
    st         = null;
  }

  /** Method resetColumnWidths()
   *  <p> This method will reset the column widths to a specific size.
   */

  private void resetColumnWidths()
  {
    for (int i=0; i<iColumns; i++)
    {
      pTable.resizeColumn(i, iColLength[i]);
    }
  }

  /* ===================================================================== *
   * -------------------------- SET METHODS ------------------------------ *
   * ===================================================================== */

  /** Method populateList()
   *  <p> This method will populate the Environment List data.
   */

  private void populateList()
  {
    int i      = 0;
    int iCount = 0;

    iChange = 0;
    iRows   = 1;

    oData = new String[1][iColumns];
    for (i=0; i<iColumns; i++)
    {
      oData[0][i] = new String("");
    }

    if (st != null)
    {
      if (st.iCount > 0)
      {
        iChange = 1;
        oData   = new Object[st.iCount][iColumns];
        iCount  = 0;
      }

      st = horizon.env.envUtility.bubbleSort(st);

      for (i=0; i<st.iCount; i++)
      {
        oData[iCount][0] = new Double(st.stItem[i].depthStart);
        oData[iCount][1] = new Double(st.stItem[i].depthEnd);
        oData[iCount][2] = new String(st.stItem[i].sDepEnv);
        oData[iCount][3] = new String(st.stItem[i].sGroup);
        oData[iCount][4] = new String(st.stItem[i].sGroup2);
        oData[iCount][5] = new String(st.stItem[i].sGroup3);
        oData[iCount][6] = new String(st.stItem[i].sDescrip);

        iCount++;
      }

      iRows = iCount;
    }
  }

  /* ===================================================================== *
   * -------------------------- SET METHODS ------------------------------ *
   * ===================================================================== */

  /** Method setData()
   *  <p> This method will reload the Environment List after adding, deleting from
   *      the formation name list.
   * @param st     = The Depositional Environment List Data Structure
   */

  public void setData(envListStruct st)
  {
    this.st = st;

    populateList();

    pTable.reload(iRows, oData);

    if (iChange == 0)
      iRows = 0;

    if (iRows > 0)
      pTable.setSelectedRow(0);
  }

  /* ===================================================================== *
   * -------------------------- GET METHODS ------------------------------ *
   * ===================================================================== */

  /** Method getTotalRows()
   *  <p> This method will return the total rows in the table.
   *  @return Total Number of Rows.
   */

  public int getTotalRows() { return (iRows); }

  /** Method getRow()
   *  <p> This method will return the selected data id in the table.
   *  @return selected data id.
   */

  public int getRow() { return (pTable.getSelectedRow()); }

  /** Method getRowData()
   *  <p> This method will return the Environment Data Structure of the
   *      selected row.
   *  @return stData = Region Data Structure.
   */

  public envStruct getRowData() { return (st.stItem[pTable.getSelectedRow()]); }

  /** Method getAllData()
   *  <p> This method will returns the Environment List Data Structure.
   *  @return st = Region List Data Structure.
   */

  public envListStruct getAllData() { return (st); }

  /** Method getScrollPane()
   *  <p> This method will return the scroll pane.
   *  @return jScrollPane - Scroll Pane containing the Table.
   */

  public JScrollPane getScrollPane() { return (pTable.getScrollPane()); }
}

/*
 *  @version 1.1 11/15/2011
 *  @author  John Victorine
 */
