/*
 * @las3Parameters.java Version 1.1 02/10/2010
 *
 * Copyright (c) 2010 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package las3;

import las3.las3ListStruct;
import las3.las3Struct;

/** Class las3Parameters
 *  <p> This Class will provide parse constants for the LAS 3 Read Class
 *
 *  @version 1.1 02/10/2010
 *  @author  John R. Victorine
 */

public class las3Parameters
{
  // ~Well Information Constants

  public static final int _START_DEPTH = 0;
  public static final int _END_DEPTH   = 1;
  public static final int _STEP_BY     = 2;
  public static final int _NULL_VALUE  = 3;

  public static final int _COMPANY     = 4;
  public static final int _WELL        = 5;
  public static final int _FIELD       = 6;
  public static final int _SECTION     = 7;
  public static final int _TOWNSHIP    = 8;
  public static final int _RANGE       = 9;
  public static final int _LOCATION    = 10;
  public static final int _LOCATION_1  = 11;
  public static final int _LOCATION_2  = 12;
  public static final int _COUNTY      = 13;
  public static final int _STATE       = 14;
  public static final int _COUNTRY     = 15;
  public static final int _PROVINCE    = 16;
  public static final int _SERVICE     = 17;
  public static final int _LICENSE     = 18;
  public static final int _DATE        = 19;
  public static final int _API         = 20;
  public static final int _UWI         = 21;
  public static final int _LATI        = 22;
  public static final int _LONG        = 23;
  public static final int _GDAT        = 24;
  public static final int _X           = 25;
  public static final int _Y           = 26;
  public static final int _HZCS        = 27;
  public static final int _UTM         = 28;
  public static final int _STATUS      = 29;
  public static final int _WELL_TOTAL  = 30;

  // ~Well Information Textfields

  public static String WELL_PARAMETERS[][] = {
    //  MNEM . UNIT   VALUE     : Description                   {Format}
      { "STRT", "F",   "0.0",     "START DEPTH",                   "F", ""},
      { "STOP", "F",   "0.0",     "END DEPTH",                     "F", ""},
      { "STEP", "F",   "0.0",     "STEP LENGTH",                   "F", ""},
      { "NULL", "",    "-999.25", "NULL VALUE",                    "F", ""},

      { "COMP", "",    "",        "Company",                       "S", ""},
      { "WELL", "",    "",        "Well Name",                     "S", ""},
      { "FLD ", "",    "",        "Field",                         "S", ""},
      { "SEC ", "",    "",        "Section",                       "I", ""},
      { "TOWN", "",    "",        "Township (e.g. 42S)",           "S", ""},
      { "RANG", "",    "",        "Range  (e.g. 25E)",             "S", ""},
      { "LOC ", "",    "",        "Location (Sec Town Range)",     "S", ""},
      { "LOC1", "",    "",        "Location 1 (quarter calls)",    "S", ""},
      { "LOC2", "",    "",        "Location 2 (footages)",         "S", ""},
      { "COUN", "",    "",        "County",                        "S", ""},
      { "STAT", "",    "",        "State",                         "S", ""},
      { "CTRY", "",    "",        "Country",                       "S", ""},
      { "PROV", "",    "",        "Province",                      "S", ""},
      { "SRVC", "",    "",        "Service Company",               "S", ""},
      { "LIC ", "",    "",        "License Number",                "S", ""},
      { "DATE", "",    "",        "Completion Date",               "DD/MM/YYYY", ""},
      { "API ", "",    "",        "API-Number",                    "S", ""},
      { "UWI ", "",    "",        "Unique Well ID Number",         "S", ""},
      { "LATI", "DEG", "",        "Latitude",                      "F", ""},
      { "LONG", "DEG", "",        "Longitude",                     "F", ""},
      { "GDAT", "",    "NAD27",   "Geodetic Datum",                "S", ""},
      { "X   ", "",    "",        "X or East-West coordinate",     "F", ""},
      { "Y   ", "",    "",        "Y or North South coordinate",   "F", ""},
      { "HZCS", "",    "UTM",     "Horizontal Co-ordinate System", "S", ""},
      { "UTM ", "",    "",        "UTM Location",                  "F", ""},
      { "STUS", "",    "",        "Well Status",                   "S", ""},
  };

  // ~Log Parameters Constants

  public static final int _PDAT   = 0;
  public static final int _APD    = 1;
  public static final int _DREF   = 2;
  public static final int _EREF   = 3;
  public static final int _RUN    = 4;

  public static final int _TDL    = 5;
  public static final int _TDD    = 6;

  public static final int _CSGL   = 7;
  public static final int _CSGD   = 8;
  public static final int _CSGS   = 9;
  public static final int _CSGW   = 10;
  public static final int _BS     = 11;

  public static final int _MUD    = 12;
  public static final int _MUDS   = 13;
  public static final int _MUDD   = 14;
  public static final int _MUDV   = 15;
  public static final int _FL     = 16;
  public static final int _PH     = 17;
  public static final int _RM     = 18;
  public static final int _RMT    = 19;
  public static final int _RMF    = 20;
  public static final int _RMFT   = 21;
  public static final int _RMC    = 22;
  public static final int _RMCT   = 23;
  public static final int _TMAX   = 24;

  public static final int _TIMC   = 25;
  public static final int _TIML   = 26;
  public static final int _T_UNIT = 27;
  public static final int _BASE   = 28;
  public static final int _ENG    = 29;
  public static final int _WIT    = 30;

  public static final int _LOG_TOTAL   = 31;

  // ~Log Parameters Textfields

  public static String LOG_PARAMETERS[][] = {
  //  MNEM . UNIT   VALUE     : Description                       {Format}
    { "PDAT", "",     "",       "Permanent Data",                  "S", "" },
    { "APD ", "F",    "",       "Above Permanent Data",            "F", "" },
    { "DREF", "",     "",       "Depth Reference (KB,DF,CB)",      "S", "" },
    { "EREF", "F",    "",       "Elevation of Depth Reference",    "F", "" },
    { "RUN ", "",     "1",      "Run Number",                      "I", "" },

    { "TDL ", "F",    "",        "Total Depth Logger",             "F", "" },
    { "TDD ", "F",    "",        "Total Depth Driller",            "F", "" },

    { "CSGL", "F",     "",       "Casing Bottom Logger",           "F", "" },
    { "CSGD", "F",     "",       "Casing Bottom Driller",          "F", "" },
    { "CSGS", "IN",    "",       "Casing Size",                    "F", "" },
    { "CSGW", "LB",    "",       "Casing Weight",                  "F", "" },

    { "BS  ", "IN",    "",       "Bit Size",                       "F", "" },

    { "MUD ", "",      "",       "Mud type",                       "S", "" },
    { "MUDS", "",      "",       "Mud Source",                     "S", "" },
    { "MUDD", "GM/CC", "",       "Mud Density",                    "F", "" },
    { "MUDV", "CC",     "",      "Mud Viscosity (Funnel)",         "F", "" },
    { "FL  ", "LB/S",  "",       "Fluid Loss",                     "F", "" },
    { "PH  ", "",      "",       "PH",                             "F", "" },
    { "RM  ", "OHM-M", "",       "Resistivity of Mud",             "F", "" },
    { "RMT ", "DEG-F", "",       "Temperature of Mud",             "F", "" },
    { "RMF ", "OHM-M", "",       "Resistivity. of Mud Filtrate",   "F", "" },
    { "RMFT", "DEG-F", "",       "Temperature of Mud Filtrate",    "F", "" },
    { "RMC ", "OHM-M", "",       "Resistivity of Mud Cake",        "F", "" },
    { "RMCT", "DEG-F", "",       "Temperature of Mud Cake",        "F", "" },
    { "TMAX", "DEG-F", "",       "Maximum Recorded Temp.",         "F", "" },

    { "TIMC", "DATE",  "",       "Date/Time Circulation Stopped",  "DD/MM/YYYY", "" },
    { "TIML", "DATE",  "",       "Date/Time Logger Tagged Bottom", "DD/MM/YYYY", "" },
    { "UNIT", "",      "",       "Logging Unit Number",            "S", "" },
    { "BASE", "",      "",       "Home Base of Logging Unit",      "S", "" },
    { "ENG ", "",      "",       "Recording Engineer",             "S", "" },
    { "WIT ", "",      "",       "Witnessed By",                   "S", "" },
  };
}

/*
 *  @version 1.1 02/10/2010
 *  @author  John Victorine
 */
