/*
 * @quakeComputePanel.java Version 1.1 05/19/2015
 *
 * Copyright (c) 2015 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package quake.gui;

import java.io.*;
import java.net.*;
import java.lang.*;
import java.util.*;
import java.util.Observable;
import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;

import cmn.cmnStruct;                // Common Data Structure

import iqstrat.iqstratHeadersStruct; // Well Header Data Structure
import event.eventFileDataStruct;    // Event File Data Structure
import quake.quakeSensorsDataStruct; // Computed Quake Location Data Structure
import quake.quakeSensorsStruct;     // Sensor Data Structure

/** Class quakeComputePanel
 *  <p> This Class will create the Table Panel
 *
 *  @version 1.1 05/19/2015
 *  @author  John R. Victorine
 */

public class quakeComputePanel extends JPanel implements ActionListener
{
  public static final int _TOP   = event.eventStandardTools._TOP;
  public static final int _BASE  = event.eventStandardTools._BASE;
  public static final int _VAc   = event.eventStandardTools._VAc;
  public static final int _VAs   = event.eventStandardTools._VAs;

  public static final int _S_No  = quake.quakeSensorsDataStruct._No;
  public static final int _S_PSt = quake.quakeSensorsDataStruct._PSt;
  public static final int _S_Vp  = quake.quakeSensorsDataStruct._Vp;
  public static final int _S_Vs  = quake.quakeSensorsDataStruct._Vs;
  public static final int _S_R   = quake.quakeSensorsDataStruct._R;

  // Input Variables

  private cmnStruct  stCMN    = null; // Common Data Structure
  private Observable notifier = null; // Observables Object

  // Global Variables

  private static final int   _SENSORS    = 0;
  private static final int   _LATITUDE   = 1;
  private static final int   _LONGITUDE  = 2;
  private static final int   _ELEVATION  = 3;
  private static final int   _DEL_PS     = 4;
  private static final int   _Vp         = 5;
  private static final int   _Vs         = 6;
  private static final int   _DISTANCE   = 7;
  private static final int   _TEXTFIELDS = 8;

  private static final int    COLUMNS    = 8;

  private static final String sCOLUMNS[] = {
	"Sensors",
	"Latitude",  "Longitude", "Elevation",
	"PS (msec)", "Vp (m/sec)", "Vs (m/sec)",  "R [m]"
  };

  public static int    TOTAL       = 3;

//  public static int    _TOTAL      = quake.quakeSensorsStruct.TOTAL;
//  public static double SENSORS[][] = quake.quakeSensorsStruct.SENSORS;

  private quakeSensorsStruct stSensor = null; // Seismic Sensor Data

  private int    iView[]   = { 0, 0, 0 };

  private int    iSensor[] = { 0,   0,   0 };
  private double dLat[]    = { 0.0, 0.0, 0.0 };
  private double dLong[]   = { 0.0, 0.0, 0.0 };
  private double rn[]      = { 0.0, 0.0, 0.0 };
  private double hn[]      = { 0.0, 0.0, 0.0 };
  private double dOut[]    = { 0.0, 0.0, 0.0 };

  public static final int _ADD    = 0;
  public static final int _MODIFY = 1;

  private int iAction = _ADD;
  private String sKEY = "0";

  private iqstratHeadersStruct stHeader = null;
  private eventFileDataStruct  stZeke   = null;
  private double dElev = 0.0;
  private double Vp    = 0.0;
  private double Vs    = 0.0;

  private int    iRows = 0;

  // Global Panel Variables

  private JTextField   txtTime    = new JTextField(); // Date/Time String
  private JTextField   txtChannel = new JTextField(); // Channel
  private JTextField   txtMag     = new JTextField(); // Magnitude of Event
  private JTextField   txtTry     = new JTextField(); // Depth (try) in elev ft
  private JTextField   txtLat     = new JTextField(); // Latitude
  private JTextField   txtLong    = new JTextField(); // Longitude
  private JTextField   txtDpth    = new JTextField(); // Depth of Event

  private JTextField   txt[][] = null;

  private JButton     btnAdd   = new JButton();
  private JButton     btnClear = new JButton();

  /** Constructor quakeComputePanel()
   *  <p> This is the Constructor for this class.
   *  @param stStruct  = Global IQSTRAT Data Structure
   *  @param pNotifier = Observable
   */

  public quakeComputePanel( cmnStruct stCMN, Observable notifier )
  {
    try
    {
      this.stCMN    = stCMN;
      this.notifier = notifier;

      jbInit();
      setButtons();
    }
    catch(Exception e)
    {
      e.printStackTrace();
    }
  }

  /** METHOD jbInit()
   *  <p> This Method will create the Frame for this class.
   *  @throws Exception
   */

  private void jbInit() throws Exception
  {
	int i,j,k;
    JPanel  pnlTop         = new JPanel();
    JPanel  pnlTWest       = new JPanel();
    JPanel  pnlTCenter     = new JPanel();

    JPanel  pnlCenter      = new JPanel();
    JPanel  panelT         = new JPanel();
    JPanel  panel          = new JPanel();
    JPanel  pnlRow[]       = null;
    JPanel  pnlRWest[]     = null;
    JPanel  pnlRCenter[]   = null;

    JPanel  pnlBase        = new JPanel();

    JPanel  pnl[][]        = null;

    JButton btns[]     = null;

    TitledBorder titledBorderTitle = new TitledBorder(
        new EtchedBorder(EtchedBorder.RAISED,
                         Color.white,
                         new Color(168, 168, 168)),
        "Seismic Event Location Calculator");
    titledBorderTitle.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    this.setLayout(new BorderLayout());
    this.setBorder(titledBorderTitle);

    // Top Panel

    //pnlTop.setLayout(new BorderLayout());

    pnlTop = buildTopPanel();

    // Center Panel

    pnlCenter.setLayout(new BorderLayout());

    // -- Header Table Button Panel

    panelT.setLayout(new BorderLayout());

    pnlTWest.setLayout(new GridLayout());
    pnlTCenter.setLayout(new GridLayout());

    panelT.add(pnlTWest,   BorderLayout.WEST);
    panelT.add(pnlTCenter, BorderLayout.CENTER);

    btns = new JButton[COLUMNS];

    for (i=0; i<COLUMNS; i++)
    {
	  btns[i] = new JButton();
	  btns[i].setText( sCOLUMNS[i] );

	  if (i==0)
	    pnlTWest.add(btns[i], null);
	  else if (i<COLUMNS)
	    pnlTCenter.add(btns[i], null);
	}

    // -- Table Panel

    panel.setLayout(new GridLayout(TOTAL,1));

    pnlRow     = new JPanel[TOTAL];
    pnlRWest   = new JPanel[TOTAL];
    pnlRCenter = new JPanel[TOTAL];
    pnl        = new JPanel[TOTAL][COLUMNS];

    txt        = new JTextField[TOTAL][_TEXTFIELDS];

    for (k=0; k<TOTAL; k++)
    {
      pnlRow[k]     = new JPanel();
      pnlRow[k].setLayout(new BorderLayout());

      pnlRWest[k]   = new JPanel();
      pnlRWest[k].setLayout(new BorderLayout());
      pnlRWest[k].setPreferredSize(btns[0].getPreferredSize());

      pnlRCenter[k] = new JPanel();
      pnlRCenter[k].setLayout(new GridLayout());

      for (j=0; j<COLUMNS; j++)
      {
		pnl[k][j] = new JPanel();
		pnl[k][j].setLayout(new BorderLayout());
	  }

      for (j=0; j<_TEXTFIELDS; j++)
      {
		txt[k][j] = new JTextField();
		if (j > _SENSORS)
		  txt[k][j].setText("0.0");
		if ((j > _SENSORS) && (j <= _ELEVATION))
		  txt[k][j].setEditable(false);
		txt[k][j].setHorizontalAlignment(JTextField.RIGHT);
		txt[k][j].addFocusListener(new quakeComputePanelFocusAdapter(this));

        pnl[k][j].add(txt[k][j],     BorderLayout.CENTER);
        if (j == 0)
          pnlRWest[k].add(pnl[k][j],   null);
        else
          pnlRCenter[k].add(pnl[k][j], null);

	  }

//	  pnlCenter.add(pnlRow[k],       null);
      panel.add(pnlRow[k],           null);
      pnlRow[k].add(pnlRWest[k],     BorderLayout.WEST);
//      pnlRWest[k].add(pnl[k][7],     null);

      pnlRow[k].add(pnlRCenter[k],   BorderLayout.CENTER);
	}

    // Base Panel

    pnlBase.setLayout(new GridLayout());

    btnAdd.setFont(new java.awt.Font("Dialog", 1, 11));
    btnAdd.setBorderPainted(true);
    btnAdd.setText("Add");
    btnAdd.addActionListener(this);

    btnClear.setFont(new java.awt.Font("Dialog", 1, 11));
    btnClear.setBorderPainted(true);
    btnClear.setText("Clear");
    btnClear.addActionListener(this);

    // Attach Widgets to Panel

    this.add(pnlTop,      BorderLayout.NORTH);

    this.add(pnlCenter,   BorderLayout.CENTER);
    pnlCenter.add(panelT, BorderLayout.NORTH);
    pnlCenter.add(panel,  BorderLayout.CENTER);

    this.add(pnlBase,     BorderLayout.SOUTH);
    pnlBase.add(btnAdd,   null);
    pnlBase.add(btnClear, null);
  }

  /** Method buildTopPanel()
   *  <p> This method will build the top panel
   */

  private JPanel buildTopPanel()
  {
	JPanel panel      = new JPanel();
	JPanel pnlTime    = new JPanel();

	JPanel pnlCenter  = new JPanel();
	JPanel pnlChannel = new JPanel();
	JPanel pnlTry     = new JPanel();
	JPanel pnlMag     = new JPanel();
	JPanel pnlLat     = new JPanel();
	JPanel pnlLong    = new JPanel();
	JPanel pnlDpth    = new JPanel();

    TitledBorder titledBorderTime = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white,new Color(165, 163, 151)),
        "MM/dd/yyyy HH:mm:ss.S");
    titledBorderTime.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderChannel = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white,new Color(165, 163, 151)),
        "Channel");
    titledBorderChannel.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderTry = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white,new Color(165, 163, 151)),
        "Depth (try) [ft]");
    titledBorderTry.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderMag = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white,new Color(165, 163, 151)),
        "Magnitude");
    titledBorderMag.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderLat = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white,new Color(165, 163, 151)),
        "Latitude");
    titledBorderLat.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderLong = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white,new Color(165, 163, 151)),
        "Longitude");
    titledBorderLong.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderDpth = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white,new Color(165, 163, 151)),
        "Depth [ft]");
    titledBorderDpth.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    panel.setLayout(new BorderLayout());

    pnlTime.setLayout(new BorderLayout());
    pnlTime.setBorder(titledBorderTime);

    txtTime.setText("");
    txtTime.setColumns(15);
	txtTime.setHorizontalAlignment(JTextField.RIGHT);
    txtTime.addFocusListener(new quakeComputePanelFocusAdapter(this));

    pnlCenter.setLayout(new GridLayout());

    pnlChannel.setLayout(new BorderLayout());
    pnlChannel.setBorder(titledBorderChannel);

    txtChannel.setText("");
	txtChannel.setHorizontalAlignment(JTextField.RIGHT);
    txtChannel.addFocusListener(new quakeComputePanelFocusAdapter(this));

    pnlMag.setLayout(new BorderLayout());
    pnlMag.setBorder(titledBorderMag);

    txtMag.setText("0.0");
	txtMag.setHorizontalAlignment(JTextField.RIGHT);
    txtMag.addFocusListener(new quakeComputePanelFocusAdapter(this));

    pnlTry.setLayout(new BorderLayout());
    pnlTry.setBorder(titledBorderTry);

    txtTry.setText("0.0");
    txtTry.setEditable(false);
	txtTry.setHorizontalAlignment(JTextField.RIGHT);
    txtTry.addFocusListener(new quakeComputePanelFocusAdapter(this));

    pnlLat.setLayout(new BorderLayout());
    pnlLat.setBorder(titledBorderLat);

    txtLat.setText("0.0");
    txtLat.setEditable(false);
	txtLat.setHorizontalAlignment(JTextField.RIGHT);
    txtLat.addFocusListener(new quakeComputePanelFocusAdapter(this));

    pnlLong.setLayout(new BorderLayout());
    pnlLong.setBorder(titledBorderLong);

    txtLong.setText("0.0");
    txtLong.setEditable(false);
	txtLong.setHorizontalAlignment(JTextField.RIGHT);
    txtLong.addFocusListener(new quakeComputePanelFocusAdapter(this));

    pnlDpth.setLayout(new BorderLayout());
    pnlDpth.setBorder(titledBorderDpth);

    txtDpth.setText("0.0");
    txtDpth.setEditable(false);
	txtDpth.setHorizontalAlignment(JTextField.RIGHT);
    txtDpth.addFocusListener(new quakeComputePanelFocusAdapter(this));

	panel.add(pnlTime,         BorderLayout.WEST);
	pnlTime.add(txtTime,       BorderLayout.CENTER);

    panel.add(pnlCenter,       BorderLayout.CENTER);
	pnlCenter.add(pnlChannel,  null);
	pnlChannel.add(txtChannel, BorderLayout.CENTER);

	pnlCenter.add(pnlMag ,     null);
	pnlMag.add(txtMag,         BorderLayout.CENTER);

	pnlCenter.add(pnlTry ,     null);
	pnlTry.add(txtTry,         BorderLayout.CENTER);

	pnlCenter.add(pnlLat ,     null);
	pnlLat.add(txtLat,         BorderLayout.CENTER);

	pnlCenter.add(pnlLong,     null);
	pnlLong.add(txtLong,       BorderLayout.CENTER);

	pnlCenter.add(pnlDpth,     null);
	pnlDpth.add(txtDpth,       BorderLayout.CENTER);

	return (panel);
  }

  /* ===================================================================== *
   * -------------------------- CLOSE DIALOG ----------------------------- *
   * ===================================================================== */

  /** Method close()
   * <p> This method will close all dialogs that are opened by this class.
   */

  public void close()
  {
    // Input Variables

    notifier   = null; // Top Observables Object
    stCMN      = null; // Common Data Structure

    stSensor   = null;

    sKEY       = null;

    if (stHeader != null)
      stHeader.delete();
    stHeader = null;

	if (stZeke != null)
	  stZeke.delete();
	stZeke = null;

    // Global Widgets

    txtTime    = null; // Date/Time String
    txtChannel = null; // Channel
    txtMag     = null; // Magnitude of Event
    txtTry     = null; // Depth (try) in elev ft
    txtLat     = null; // Latitude
    txtLong    = null; // Longitude
    txtDpth    = null; // Depth of Event

    for (int i=0; i<TOTAL; i++)
    {
	  for (int j=0; j<COLUMNS; j++)
	  {
        txt[i][j] = null;
	  }
	}
	txt = null;

    btnAdd   = null;
    btnClear = null;
  }

  /* ===================================================================== *
   * -------------------------- GET METHODS ------------------------------ *
   * ===================================================================== */

  /** Method getData()
   *  <p> This method will retrieve the data necessary to compute the depth
   *      of the seismic event
   */

  private void getData()
  {
	int    iCount = 0;
//	double data   = 0.0;

//    for (int i=0; i<TOTAL; i++)
//    {
//	  data = cmn.cmnString.stringToDouble(txt[i][_DISTANCE].getText());
//	  if (data > 0.0)
//		iView[i] = 1;
//	  else if (data == 0.0)
//	    iView[i] = 0;
//    }

    for (int i=0; i<TOTAL; i++)
    {
	  if (iView[i] == 1)
	  {
		if (iCount < 3)
	    {
		  iSensor[iCount] = i+1;
  	      dLat[iCount] = cmn.cmnString.stringToDouble(txt[i][_LATITUDE].getText());
	      dLong[iCount] = cmn.cmnString.stringToDouble(txt[i][_LONGITUDE].getText());
	      hn[iCount] = 0.3048 * cmn.cmnString.stringToDouble(txt[i][_ELEVATION].getText());
	      rn[iCount] = cmn.cmnString.stringToDouble(txt[i][_DISTANCE].getText());

//System.out.println(iSensor[iCount]+" "+dLat[iCount]+" "+dLong[iCount]+" "+hn[iCount]+" "+rn[iCount]);
	      iCount++;
		}
	  }
	}

    compute();
  }

  /** Method getVelocities()
   *  <p> This method will retrieve the Average Vs and Vp from the
   *      event file data structure.
   */

  private void getVelocities()
  {
    double dStart = 0.0;
    double dEnd   = 0.0;
    double depth  = 0.0;

    depth = cmn.cmnString.stringToDouble( txtTry.getText() );
//System.out.println("DEPTH "+depth+" "+dElev);

	if (stZeke != null)
	{
	  if ((stZeke.iVAc > -1) && (stZeke.iVAs > -1))
	  {
        for (int i=0; i<stZeke.iRows; i++)
        {
		  dStart = dElev - stZeke.getRow(_TOP, i);
		  dEnd   = dElev - stZeke.getRow(_BASE, i);

		  if ((depth < dStart) && (depth >= dEnd))
		  {
            Vp = stZeke.getRow(_VAc, i);
            Vs = stZeke.getRow(_VAs, i);
//System.out.println(depth+" "+dStart+" "+dEnd+" <> "+Vp+" "+Vs);
	      }
		}
	  }
	}
  }

  /** Method getComputedData()
   *  <p> This method will retrieve the computed data for saving
   *  @return st = Computed data structure
   */

  public quakeSensorsDataStruct getComputedData()
  {
	quakeSensorsDataStruct st = new quakeSensorsDataStruct();

	if (iAction == _ADD)
	{
	  sKEY = cmn.cmnString.UniqueName();
	}

    // Time and Computed location of seismic event.

    st.sTime      = new String( txtTime.getText() );
    st.sChannel   = new String( txtChannel.getText() );
    st.dMagnitude = cmn.cmnString.stringToDouble( txtMag.getText() );
    st.dLatitude  = cmn.cmnString.stringToDouble( txtLat.getText() );
    st.dLongitude = cmn.cmnString.stringToDouble( txtLong.getText() );
    st.depth      = cmn.cmnString.stringToDouble( txtDpth.getText() );

    for (int i=0; i<TOTAL; i++)
    {
      st.data[i][_S_No]  = cmn.cmnString.stringToDouble( txt[i][_SENSORS].getText() );
      st.data[i][_S_PSt] = cmn.cmnString.stringToDouble( txt[i][_DEL_PS].getText() );
      st.data[i][_S_Vp]  = cmn.cmnString.stringToDouble( txt[i][_Vp].getText() );
      st.data[i][_S_Vs]  = cmn.cmnString.stringToDouble( txt[i][_Vs].getText() );
      st.data[i][_S_R]   = cmn.cmnString.stringToDouble( txt[i][_DISTANCE].getText() );
	}

	return (st);
  }

  /* ===================================================================== *
   * -------------------------- SET METHODS ------------------------------ *
   * ===================================================================== */

  /** Method setButtons()
   *  <p> This method will enable or disable buttons depending on the select
   *      data.
   */

  private void setButtons()
  {
	txtTry.setEditable(false);
    btnAdd.setEnabled(false);

	if (iRows > 0)
	{
  	  txtTry.setEditable(true);
	}

	if (txtTime.getText().length() > 0)
	{
      btnAdd.setEnabled(true);
	}
  }

  /** Method setSensors()
   *  <p> This method will set the sensor data
   *  @param st = The Sensor data structure
   */

  public void setSensors( quakeSensorsStruct st ) { this.stSensor = st; }

  /** Method setData()
   *  <p> This method will set the event file data Structure
   *  @param stH = The Well Header Data Structure
   *  @param st  = The Event File Data Structure
   */

  public void setData( iqstratHeadersStruct stH, eventFileDataStruct st )
  {
	this.stHeader = stH;
	this.stZeke   = st;

	if (stHeader != null)
	{
      if ( stH.dGL > 0.0 )
        dElev = stH.dGL;
      else if ( stH.dKB > 0.0 )
        dElev = stH.dKB;
      else if ( stH.dDF > 0.0 )
        dElev = stH.dDF;
	}
  }

  /** Method setRows()
   *  <p> This method will check the number of rows
   *  @param iRows = the total number of wells in list from wells panel
   */

  public void setRows(int iRows)
  {
	this.iRows = iRows;
	setButtons();
  }

  /** Method setComputedData()
   *  <p> This method will retrieve the computed data for saving
   *  @return st = Computed data structure
   */

  public void setComputedData(quakeSensorsDataStruct st)
  {
	int id = 0;

	if (st != null)
	{
      iAction = _MODIFY;
      sKEY    = new String( st.sKEY );

      // Time and Computed location of seismic event.

      txtTime.setText( st.sTime );
      txtChannel.setText( st.sChannel );
      txtMag.setText(""  + st.dMagnitude);
      txtLat.setText(""  + st.dLatitude);
      txtLong.setText("" + st.dLongitude);
      txtDpth.setText("" + st.depth);

      for (int i=0; i<TOTAL; i++)
      {
        txt[i][_SENSORS].setText("" + (int) st.data[i][_S_No]);
        id = (int) cmn.cmnString.stringToInt(txt[i][_SENSORS].getText());

        if (stSensor != null)
        {
	      for (int j=0; j<stSensor.iTotal; j++)
	      {
	        if (id == stSensor.id[j])
	        {
              txt[i][_LATITUDE].setText(""  + stSensor.dLat[j]);
	          txt[i][_LONGITUDE].setText("" + stSensor.dLong[j]);
	          txt[i][_ELEVATION].setText("" + stSensor.dElev[j]);
		    }
		  }
		}

        txt[i][_DEL_PS].setText("" + st.data[i][_S_PSt]);
        txt[i][_Vp].setText("" + st.data[i][_S_Vp]);
        txt[i][_Vs].setText("" + st.data[i][_S_Vs]);
        txt[i][_DISTANCE].setText("" + st.data[i][_S_R]);
	  }
	}
  }

  /** Method clear()
   *  <p> This method will clear the text fields
   */

  private void clear()
  {
    iAction = _ADD;
    sKEY    = new String( "0" );

    // Time and Computed location of seismic event.

    txtTime.setText( "" );
    txtChannel.setText( "" );
    txtMag.setText("0.0");
    txtLat.setText("0.0");
    txtLong.setText("0.0");
    txtDpth.setText("0.0");

    for (int i=0; i<TOTAL; i++)
    {
      txt[i][_SENSORS].setText("");

      txt[i][_LATITUDE].setText("0.0");
      txt[i][_LONGITUDE].setText("0.0");
      txt[i][_ELEVATION].setText("0.0");

      txt[i][_DEL_PS].setText("0.0");
      txt[i][_Vp].setText("0.0");
      txt[i][_Vs].setText("0.0");
      txt[i][_DISTANCE].setText("0.0");
    }

    if (notifier != null)
      notifier.notifyObservers( new String("Clear Edit") );
  }

  /* ===================================================================== *
   * ---------------------------- COMPUTE -------------------------------- *
   * ===================================================================== */

  /** Method compute()
   *  <p> This method will compute the Location of the Seismic Event in
   *      UTMx, UTMy, and Elevation (m).
   */

  private void compute()
  {
	int i,j;
    double q       = 0.0;
    double dlat    = 0.0;
    double dlong   = 0.0;

    if (isReady())
    {
      dOut = quake.math.quakeMath.compute(dLat, dLong, rn, hn);

      dlat  = dOut[0];
      dlong = dOut[1];
      q     = dOut[2];

      q = 3.2808399 * q;
      q = ( (int) (q * 1000.0) ) / 1000.0;

      txtLat.setText(""  + dlat);
      txtLong.setText("" + dlong);
      txtDpth.setText("" + q);

 	  txtDpth.setForeground(Color.black);
      if ( (int) cmn.cmnString.stringToDouble(txt[0][_ELEVATION].getText()) ==
           (int) cmn.cmnString.stringToDouble(txtDpth.getText()))
      {
		txtDpth.setForeground(Color.red);
	  }
	  else if (dlat == dlong)
	    txtDpth.setForeground(Color.black);
	  else
		txtDpth.setForeground(Color.blue);

//      System.out.println(dlat+" "+dlong+" "+q);
	}
  }

  /** Method isReady()
   *  <p> This method will check that all the fields are filled so the
   *      location of the seismic event can be computed.
   */

  private boolean isReady()
  {
	boolean bValue = false;

    if ((rn[0] > 0.0) && (rn[2] > 0.0) && (rn[2] > 0.0))
    {
	  bValue = true;
	}

	return (bValue);
  }

  /* ===================================================================== *
   * ---------------------------- ACTIONS -------------------------------- *
   * ===================================================================== */

  /** Method add()
   *  <p> This method will move the data to the quake location list table
   */

  private void add()
  {
    if (notifier != null)
      notifier.notifyObservers( new String("Add Quake Data") );
  }

  /** METHOD actionPerformed()
   *  <p> This Method will handle all the actions within the Frame.
   *  @param event - Action Event
   */

  public void actionPerformed(ActionEvent event)
  {
    if (event.getSource() == btnAdd)   { add(); }    // Add
    if (event.getSource() == btnClear) { clear(); }  // Clear

	setButtons();
  }

  /** METHOD focusLost()
   *  <p> This Method will handle the lost focus event for the text fields.
   *  @param e      = Focus Lost Event
   */

  protected void focusLost(FocusEvent e)
  {
	int    i,j;
    int    iNumeric = 0;
    String sMessage = new String("");
    String sTemp    = new String("");
    int    id       = -1;

    if (e.getSource() == txtTime) // Date/Time String
    {
	}

    if (e.getSource() == txtChannel) // Channel
    {
	}

    if (e.getSource() == txtTry ) // Depth (try) in elev ft
    {
      iNumeric = 1;
      sTemp    = txtTry.getText();
      sMessage = new String("The Depth Value is a Numeric Field");
	}

    if (e.getSource() == txtMag ) // Magnitude
    {
      iNumeric = 1;
      sTemp    = txtLat.getText();
      sMessage = new String("The Magnitude Value is a Numeric Field");
	}

    if (e.getSource() == txtLat ) // Latitude
    {
      iNumeric = 1;
      sTemp    = txtLat.getText();
      sMessage = new String("The Latitude Value is a Numeric Field");
	}

    if (e.getSource() == txtLong) // Longitude
    {
      iNumeric = 1;
      sTemp    = txtLong.getText();
      sMessage = new String("The Longitude Value is a Numeric Field");
	}

    if (e.getSource() == txtDpth) // Depth of Event
    {
      iNumeric = 1;
      sTemp    = txtDpth.getText();
      sMessage = new String("The Depth Value is a Numeric Field");
	}

    for (i=0; i<TOTAL; i++)
    {
	  for (j=0; j<COLUMNS; j++)
	  {
	    if (e.getSource() == txt[i][j])
  	    {
          iNumeric = 1;
          sTemp    = txt[i][j].getText();
//System.out.println(i+" "+j+" "+sTemp);

          switch(j)
          {
			case _SENSORS:
			  id = _SENSORS;
			  break;
			case _LATITUDE:
              sMessage = new String("The Latitude Value is a Numeric Field");
              break;
			case _LONGITUDE:
              sMessage = new String("The Longitude Value is a Numeric Field");
              break;
			case _ELEVATION:
              sMessage = new String("The Elevation Value is a Numeric Field");
              break;
			case _DEL_PS:
              sMessage = new String("The Time Difference Value is a Numeric Field");
              break;
			case _Vp:
              sMessage = new String("The Average Velocity Value is a Numeric Field");
              break;
			case _Vs:
              sMessage = new String("The Average Velocity Value is a Numeric Field");
              break;
			case _DISTANCE:
              sMessage = new String("The Distance Value is a Numeric Field");
              break;
		  }
		}
	  }
	}

    if (iNumeric == 1)
    {
      if (!cmn.cmnString.isNumeric(sTemp))
      {
		if (id == -1)
		{
          JOptionPane.showMessageDialog((Component) null,
                                         sMessage,
                                         "ERROR",
                                         JOptionPane.ERROR_MESSAGE);
	    }

        if (e.getSource() == txtTry ) // Depth (try) in elev ft
        {
          txtTry.setText("0.0");
	    }

        if (e.getSource() == txtMag ) // Magnitude
        {
		  txtMag.setText("0.0");
		}

        if (e.getSource() == txtLat ) // Latitude
        {
          txtLat.setText("0.0");
	    }

        if (e.getSource() == txtLong) // Longitude
        {
          txtLong.setText("0.0");
	    }

        if (e.getSource() == txtDpth) // Depth of Event
        {
          txtDpth.setText("0.0");
  	    }

        for (i=0; i<TOTAL; i++)
        {
		  if (e.getSource() == txt[i][0])
		  {
  		    iView[i] = 0;
			txt[i][0].setText("");

 	        for (j=1; j<COLUMNS; j++)
	        {
              txt[i][j].setText("0.0");
			}
		  }

	      for (j=1; j<COLUMNS; j++)
	      {
	        if (e.getSource() == txt[i][j])
  	        {
		      if (sTemp.length() > 0)
			    txt[i][j].setText("0.0");
			}
          }
		}
	  }
	  else
	  {
        checkWellData();       // Does Well Velocity & Time Exists.
        loadVelocities(e);     // Load Average Velocities by depth
        loadSensorLocation(e); // Load Sensor Location Data
        checkDistance();       // Compute Seismic Event Location
	  }
	}

	setButtons();
  }

  /** Method checkWellData()
   *  @param e = Focus Lost Event
   */

  private void loadVelocities(FocusEvent e)
  {
	double data = 0.0;
	double R    = 0.0;

    getVelocities();

    for (int i=0; i<TOTAL; i++)
    {
      if (e.getSource() == txt[i][_DEL_PS])
	  {
		data = cmn.cmnString.stringToDouble( txt[i][_DEL_PS].getText() );
		if (data > 0.0)
		{
          txt[i][_Vp].setText("" + Vp);
          txt[i][_Vs].setText("" + Vs);

          if ((Vp > 0.0) && (Vs > 0.0))
          {
            R = quake.math.quakeMath.computeDistance( data, Vp, Vs );
            txt[i][_DISTANCE].setText("" + R);
		  }
		}
	  }
	}
  }

  /** Method checkWellData()
   */

  private void checkWellData()
  {
	if (stZeke == null)
	{
      if (notifier != null)
        notifier.notifyObservers( new String("Get Well Velocity & Time Data") );
    }
  }

  /** Method checkDistance()
   *  <p> This method will automatically check to see if all distances are
   *      loaded and compute the location of the event.
   */

  private void checkDistance()
  {
	double data = 0.0;
	int    iSum = 0;

    for (int i=0; i<TOTAL; i++)
    {
	  data = cmn.cmnString.stringToDouble(txt[i][_DISTANCE].getText());

	  if (data > 0.0)
	  {
		iView[i] = 1;
		iSum = iSum + 1;
	  }
	  else if (data == 0.0)
	    iView[i] = 0;
    }

    if (iSum == 3)
    {
  	  getData();
	}
	else
	{
      txtLat.setText("0.0");
      txtLong.setText("0.0");
      txtDpth.setText("0.0");
	}
  }

  /** Method loadSensorLocation()
   *  <p> This method will automatically load the location data for the sensor
   *  @param e = Focus Lost Event
   */

  private void loadSensorLocation(FocusEvent e)
  {
	int id = -1;

    for (int i=0; i<TOTAL; i++)
    {
      if (e.getSource() == txt[i][_SENSORS])
      {
        id = (int) cmn.cmnString.stringToInt(txt[i][0].getText());

        if (stSensor != null)
        {
	      for (int j=0; j<stSensor.iTotal; j++)
	      {
	        if (id == stSensor.id[j])
	        {
              txt[i][_LATITUDE].setText(""  + stSensor.dLat[j]);
	          txt[i][_LONGITUDE].setText("" + stSensor.dLong[j]);
	          txt[i][_ELEVATION].setText("" + stSensor.dElev[j]);
		    }
		  }
	    }
	  }
	}
  }
}

/** CLASS quakeComputePanelFocusAdapter()
  *  <p> This Class will handle Actions when focus is lost on a textfield
  */

class quakeComputePanelFocusAdapter extends java.awt.event.FocusAdapter
{
  quakeComputePanel adaptee;

  quakeComputePanelFocusAdapter(quakeComputePanel adaptee)
  {
    this.adaptee = adaptee;
  }

  public void focusLost(FocusEvent e) { adaptee.focusLost(e); }
}

/*
 *  @version 1.1 05/19/2015
 *  @author  John Victorine
 */
