/*
 * @quakeEventPanel.java Version 1.1 05/19/2015
 *
 * Copyright (c) 2012 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package quake.gui;

import java.io.*;
import java.net.*;
import java.lang.*;
import java.util.*;
import java.util.Observable;
import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.event.*;

import iqstrat.iqstratStruct;      // Global IQSTRAT Data Structure

import quake.gui.quakeEventTable;  // Zonation Table
import event.eventFileDataStruct;  // Zonation File Data Structure

/** Class quakeEventPanel
 *  <p> This Class will create the Table Panel.
 *
 *  @version 1.1 05/19/2015
 *  @author  John R. Victorine
 */

public class quakeEventPanel extends JPanel implements ActionListener
{
  // Input Variables

  private iqstratStruct       stStruct = null; // Global IQSTRAT Data Structure
  private eventFileDataStruct stData   = null; // Zonation File Data Structure

  // Global Variables

  private quakeEventTable     pTable   = null; // Zonation Data Table

  // Global Frame Widgets

  /** Constructor quakeEventPanel()
   *  <p> This is the Constructor for this class.
   *  @param stStruct  = Global IQSTRAT Data Structure
   */

  public quakeEventPanel( iqstratStruct stStruct, eventFileDataStruct stData )
  {
    try
    {
      this.stStruct = stStruct;
      this.stData   = stData;

      jbInit();
    }
    catch(Exception e)
    {
      e.printStackTrace();
    }
  }

  /** METHOD jbInit()
   *  <p> This Method will create the Frame for this class.
   *  @throws Exception
   */

  private void jbInit() throws Exception
  {
    JPanel pnlTop      = new JPanel();
    JLabel lbl         = new JLabel();
    JScrollPane scroll = new JScrollPane();

    this.setLayout(new BorderLayout());

    // Top Panel

    pnlTop.setLayout(new BorderLayout());

    lbl.setText("p=compression; s=shear; sf=fast shear; ss=slow shear;    " +
                "[Edit] - Editable Column");
    lbl.setForeground(Color.blue);
    lbl.setFont(new java.awt.Font("Monospaced", 2, 11));

    // Center Panel

    pTable = new quakeEventTable(stStruct, stData);
    scroll = pTable.getScrollPane();

    this.add(pnlTop, BorderLayout.NORTH);
    pnlTop.add(lbl,  BorderLayout.WEST);

    this.add(scroll, BorderLayout.CENTER);
  }

  /* ===================================================================== *
   * -------------------------- CLOSE DIALOG ----------------------------- *
   * ===================================================================== */

  /** Method close()
   * <p> This method will close all dialogs that are opened by this class.
   */

  public void close()
  {
    stStruct  = null; // Global IQSTRAT Data Structure
    stData    = null; // Zonation File Data Structure
    pTable    = null; // Zonation Data Table
  }

  /* ===================================================================== *
   * -------------------------- SET METHODS ------------------------------ *
   * ===================================================================== */

  /** Method setData()
   * <p> This method will set the Zonation File Data Structure
   * @param st = the Zonation File Data Structure
   */

  public void setData( eventFileDataStruct st) { this.stData = st; }

  /* ===================================================================== *
   * ---------------------------- ACTIONS -------------------------------- *
   * ===================================================================== */

  /** METHOD actionPerformed()
   *  <p> This Method will handle all the actions within the Frame.
   *  @param event - Action Event
   */

  public void actionPerformed(ActionEvent event)
  {
  }
}


/*
 *  @version 1.1 05/19/2015
 *  @author  John Victorine
 */

