/*
 * @quakeFrame.java Version 1.1 01/30/2015
 *
 * Copyright (c) 2015 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package quake.gui;

import java.util.Observer;
import java.util.Observable;
import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;

import java.math.BigDecimal;

import cmn.cmnStruct;                  // Common Data Structure
import iqstrat.iqstratStruct;          // IQSTRAT Data Structure

import quake.gui.quakeWellPanel;       // Well Data List Panel
import quake.gui.quakeComputePanel;    // Compute quake location Panel
import quake.gui.quakeSensorDataPanel; // List of Computed quake locations
import quake.io.ReadSensorXMLFile;     // Read Sensor Location Data from DB
import quake.quakeSensorsStruct;       // Sensor Data Structure

import event.eventXDataStruct;         // Event X Data Structure

//import quake.gui.quakeTablePanel;   // Quake Sensor Table Panel
//import quake.gui.quakeSeismicPanel; // Seismic Daata from Sensors

/** Class quakeFrame
 *  <p> This Class will allow the user to enter the Seismic Stations by
 *      number and enter the radius of the event from the station.  The
 *      program will then compute the X,Y,Z in UTM coordinates plus Z as
 *      depth.
 *
 *  @version 1.1 1/16/2008
 *  @author  John R. Victorine
 */

public class quakeFrame extends JFrame implements ActionListener, Observer
{
  // Input Variables

  private Observable    notifier = null;
  private cmnStruct     stCMN    = null;
  private iqstratStruct stStruct = null;

  // Global Variables

  private quakeWellPanel       pnlWells = null;
  private quakeComputePanel    pnlTable = null;
  private quakeSensorDataPanel pnlList  = null;

  private quakeSensorsStruct   stSensor = null;

//  private quakeTablePanel   pnlTable   = null;
//  private quakeSeismicPanel pnlSeismic = null;

  // Global Frame Variables

  private JButton btnClose = null;
  private JButton btn2D    = null;
  private JButton btn3D    = null;

  /** Constructor quakeFrame()
   *  <p> This is the Constructor for this class.
   *  @param stCMN = Common Global Data Structure
   */

  public quakeFrame(iqstratStruct stStruct)
  {
    try
    {
	  this.stStruct = stStruct;
      this.stCMN    = stStruct.stCMN;

      jbInit();

      read();
      pnlTable.setSensors( stSensor );
      pnlList.setSensors( stSensor );

      addWindowListener( new quakeFrame_WindowListener() );
    }
    catch(Exception e)
    {
      e.printStackTrace();
    }
  }

  /** METHOD jbInit()
   *  <p> This Method will create the Frame for this class.
   *  @throws Exception
   */

  private void jbInit() throws Exception
  {
	JPanel pnlTop     = new JPanel();
	JPanel pnlButtons = new JPanel();
	JPanel pnlMenu    = new JPanel();

//	JPanel pnlSeismic = new JPanel();
	JPanel pnlCenter  = new JPanel();
	JPanel pnlBase    = new JPanel();

    JButton btnI      = new JButton();
    JButton btnII     = new JButton();

    // Build Observable

    notifier = new quakeFrameObservable();

    // Add this Dialog to the notifier as observer

    notifier.addObserver(this);

    // Build Frame

    this.setTitle("Enter Seismic Information");
    this.getContentPane().setLayout(new BorderLayout());

    // Top Panel

    pnlTop.setLayout(new BorderLayout());

    // Buttons

    pnlButtons.setLayout(new BorderLayout());

    btnClose = new JButton(createImageIcon(cmn.cmnStruct.CLOSE));
    btnClose.setPreferredSize(new Dimension(27, 27));
    btnClose.setToolTipText("Close Data");
    btnClose.addActionListener(this);

    btnI.setPreferredSize(new Dimension(4, 35));
    btnI.setText("");

    btn2D = new JButton(createImageIcon(cmn.cmnStruct.D2));
    btn2D.setPreferredSize(new Dimension(27, 27));
    btn2D.setToolTipText("2D Plot");
    btn2D.addActionListener(this);

    btn3D = new JButton(createImageIcon(cmn.cmnStruct.D3));
    btn3D.setPreferredSize(new Dimension(27, 27));
    btn3D.setToolTipText("3D Plot");
    btn3D.addActionListener(this);

    btnII.setPreferredSize(new Dimension(4, 35));
    btnII.setText("");

    // -- Header information Panel

    pnlWells = new quakeWellPanel( stStruct, notifier );

    // Center Panel

    pnlCenter.setLayout(new BorderLayout());

    // -- Sensor Table Panel

    pnlTable = new quakeComputePanel( stCMN, notifier );

    pnlList  = new quakeSensorDataPanel( stCMN, notifier );

    // Attach Widgets to Frame

    this.getContentPane().add(pnlTop,    BorderLayout.NORTH);
    pnlTop.add(pnlButtons,               BorderLayout.NORTH);
    pnlButtons.add(pnlMenu,              BorderLayout.WEST);
    pnlMenu.add(btnClose,                null);
//    pnlMenu.add(btnI,                   null);
//    pnlMenu.add(btn2D,                  null);
//    pnlMenu.add(btn3D,                  null);
//    pnlMenu.add(btnII,                  null);

    pnlTop.add(pnlWells,                 BorderLayout.CENTER);

    this.getContentPane().add(pnlCenter, BorderLayout.CENTER);
    pnlCenter.add(pnlTable,              BorderLayout.NORTH);
    pnlCenter.add(pnlList,               BorderLayout.CENTER);

    // Display the Frame

    this.setSize(new Dimension(810, 575));
    Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
    this.setLocation((d.width - this.getSize().width) / 2,
                     (d.height - this.getSize().height) / 2);

    this.setResizable(false);
    this.setVisible(true);
  }

  /** Method buildSensorTable()
   * <p> This method will build a Sensor Location Table
   */

  public JPanel buildSensorTable()
  {
	JPanel panel         = new JPanel();

	return (panel);
  }

  /** METHOD createImageIcon()
   *  <p> This Method will create the Frame for this class.
   *  @param path = the path to the image
   *  @return image icon
   */

  protected ImageIcon createImageIcon(String path)
  {
    java.net.URL imgURL = getClass().getResource(path);
    ImageIcon    image  = null;

    if (imgURL != null)
    {
      image = new ImageIcon(imgURL);
    }
    else
    {
       System.err.println("Couldn't find file: " + path);
    }

    return (image);
  }

  /* ===================================================================== *
   * ----------------------- READ SENSOR METHOD -------------------------- *
   * ===================================================================== */

  /** Method read()
   *  <p> This method will read the Sensor information from the KGS DB
   */

  private void read()
  {
    ReadSensorXMLFile pRead = new ReadSensorXMLFile(quake.io.ReadSensorXMLFile.URL);
    stSensor = pRead.Process(cmn.cmnStruct.GET_SENSORS);
  }

  /* ===================================================================== *
   * ------------------------- CLOSE METHODS ----------------------------- *
   * ===================================================================== */

  /** METHOD quakeFrame_WindowListener()
   *  <p> This Class will handle Frame Close Action using the X Button on Frame
   */

  public class quakeFrame_WindowListener extends WindowAdapter
  {
    public void windowClosing(WindowEvent e)
    {
      close();
//      System.exit(0);
    }
  }

  /** Method close()
   * <p> This method will close all dialogs that are opened by this class.
   */

  public void close()
  {
    notifier = null;
    stCMN    = null;
    stStruct = null;

    if (stSensor != null)
      stSensor.delete();
    stSensor = null;

    if (pnlWells != null)
      pnlWells.close();
    pnlWells   = null;

    if (pnlTable != null)
      pnlTable.close();
    pnlTable   = null;

    if (pnlList != null)
      pnlList.close();
    pnlList  = null;

    dispose();
  }

  /* ===================================================================== *
   * ------------------------ COMPUTE LOCATION --------------------------- *
   * ===================================================================== */


  /* ===================================================================== *
   * ---------------------------- ACTIONS -------------------------------- *
   * ===================================================================== */

  /** METHOD actionPerformed()
   *  <p> This Method will handle all the actions within the Frame.
   *  @param event - Action Event
   */

  public void actionPerformed(ActionEvent event)
  {
	if (event.getSource() == btnClose) { close(); }
  }

  /** METHOD focusLost()
   *  <p> This Method will handle the lost focus event for the text fields.
   *  @param e      = Focus Lost Event
   */

  protected void focusLost(FocusEvent e)
  {
  }

  /** Method update()
   * <p> This method will handle the Observable Actions
   * @param obj = The Observable making the call
   * @param arg = The returned object
   */

  public void update(Observable obj, Object arg)
  {
    String sArg      = new String((String) arg);

    if (sArg.equals("Get Well Table Rows"))
    {
	  if ((pnlWells != null) && (pnlTable != null))
	  {
		pnlTable.setRows(pnlWells.getTotalRows());
	  }
	}

    if (sArg.equals("Get Well Velocity & Time Data"))
    {
	  if ((pnlWells != null) && (pnlTable != null))
      {
        eventXDataStruct stX = pnlWells.getEventData();
        if (stX != null)
        {
          pnlTable.setData( iqstrat.iqstratHeadersUtility.copy(stX.stHeader),
                            event.eventFileDataUtility.copy(stX.stZeke) );
		}
	  }
	}

    if (sArg.equals("Add Quake Data"))
    {
	  if ((pnlList != null) && (pnlTable != null))
	  {
		pnlList.setData( pnlTable.getComputedData() );
	  }
	}

    if (sArg.equals("Modify Quake Data"))
    {
	  if ((pnlList != null) && (pnlTable != null))
	  {
	    pnlTable.setComputedData( pnlList.getData() );
	  }
	}

	if (sArg.equals("Clear Edit"))
	{
	  if (pnlList != null)
	  {
	    pnlList.clear();
	  }
	}
  }
}

/** CLASS quakeFrameFocusAdapter()
 *  <p> This Class will handle Actions when focus is lost on a textfield
 */

class quakeFrameFocusAdapter extends java.awt.event.FocusAdapter
{
  quakeFrame adaptee;

  quakeFrameFocusAdapter(quakeFrame adaptee)
  {
    this.adaptee = adaptee;
  }

  public void focusLost(FocusEvent e) { adaptee.focusLost(e); }
}

/** CLASS quakeFrameObservable()
 *  <p> This Class will handle Observable Actions when action occurs on a
 *      frame or panel
 */

class quakeFrameObservable extends Observable
{
  public void notifyObservers(Object b)
  {
    setChanged();
    super.notifyObservers(b);
  }
}

/*
 *  @version 1.1 01/30/2015
 *  @author  John Victorine
 */
