/*
 * @quakeSeismicPanel.java Version 1.1 05/12/2015
 *
 * Copyright (c) 2015 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package quake.gui;

import java.io.*;
import java.net.*;
import java.lang.*;
import java.util.*;
import java.util.Observable;
import java.awt.*;
import java.awt.event.*;
import javax.swing.border.*;

import javax.swing.*;
import javax.swing.event.*;

import cmn.cmnStruct;               // Common Data Structure
import quake.gui.quakeSeismicTable; // Sensor Data List Table

/** Class quakeSeismicPanel
 *  <p> This Class will create the Zonation Well Data Panel
 *
 *  @version 1.1 05/12/2015
 *  @author  John R. Victorine
 */

public class quakeSeismicPanel extends JPanel implements ActionListener
{
  // Input Variables

  private cmnStruct  stCMN    = null; // Common Data Structure
  private Observable notifier = null; // Observables Object

  // Global Variables

  private quakeSeismicTable pnlWell  = null;

  // Global Frame Widgets

  private JButton     btnSeismic   = null;  // Seismic Button

  private JButton     btnModify    = new JButton(); // Modify
  private JButton     btnDelete    = new JButton(); // Delete
  private JButton     btnDeleteAll = new JButton(); // Delete All

  /** Constructor quakeSeismicPanel()
   *  <p> This is the Constructor for this class.
   *  @param stCMN     = Comman Data Structure
   *  @param pNotifier = Observable
   */

  public quakeSeismicPanel( cmnStruct stCMN, Observable notifier )
  {
    try
    {
      this.stCMN    = stCMN;
      this.notifier = notifier;

      jbInit();
    }
    catch(Exception e)
    {
      e.printStackTrace();
    }
  }

  /** METHOD jbInit()
   *  <p> This Method will create the Frame for this class.
   *  @throws Exception
   */

  private void jbInit() throws Exception
  {
	JPanel      pnlIcon   = new JPanel();
	JPanel      pnlCenter = new JPanel();
	JPanel      pnlTable  = new JPanel();
	JPanel      pnlbtns   = new JPanel();

    JScrollPane scroll    = new JScrollPane();

    TitledBorder titledBorderTitle = new TitledBorder(
        new EtchedBorder(EtchedBorder.RAISED,
                         Color.white,
                         new Color(168, 168, 168)),
        "P-S Wave Time Difference from KGS Seismic Sensors");
    titledBorderTitle.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderWAVE = new TitledBorder(
        BorderFactory.createEmptyBorder(), "Time CSV");
    titledBorderWAVE.setTitleJustification(TitledBorder.CENTER);
    titledBorderWAVE.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    this.setLayout(new BorderLayout());
    this.setPreferredSize( new Dimension(10, 150));
    this.setBorder( titledBorderTitle );

    // Zonation ICON Panel

    pnlIcon.setBackground(Color.white);
    pnlIcon.setBorder( titledBorderWAVE );

    btnSeismic = new JButton( createImageIcon( cmn.cmnStruct.WAVE_ICON ) );
    btnSeismic.setBackground(Color.white);
    btnSeismic.setPreferredSize( new Dimension(64, 64) );
    btnSeismic.addActionListener( this );

    // Center Panel

    pnlCenter.setLayout(new BorderLayout());

    // -- Well List Table Panel

    pnlWell = new quakeSeismicTable();
    scroll  = pnlWell.getScrollPane();

    // -- Buttons Panel

    pnlbtns.setLayout(new GridLayout());

    btnModify.setFont(new java.awt.Font("Dialog", 1, 11));
    btnModify.setBorderPainted(true);
    btnModify.setText("Modify Row");
    btnModify.addActionListener(this);

    btnDelete.setFont(new java.awt.Font("Dialog", 1, 11));
    btnDelete.setBorderPainted(true);
    btnDelete.setText("Delete Row");
    btnDelete.addActionListener(this);

    btnDeleteAll.setFont(new java.awt.Font("Dialog", 1, 11));
    btnDeleteAll.setBorderPainted(true);
    btnDeleteAll.setText("Delete All Rows");
    btnDeleteAll.addActionListener(this);

    // Attach Widgets to Panel

    this.add( pnlIcon,         BorderLayout.WEST );
    pnlIcon.add( btnSeismic,   null);

    this.add(pnlCenter,        BorderLayout.CENTER );
    pnlCenter.add( scroll,     BorderLayout.CENTER );
    pnlCenter.add( pnlbtns,    BorderLayout.SOUTH );
    pnlbtns.add( btnModify,    null );
    pnlbtns.add( btnDelete,    null );
    pnlbtns.add( btnDeleteAll, null );
  }

  /** METHOD createImageIcon()
   *  <p> This Method will create the Frame for this class.
   *  @param path = the path to the image
   *  @return image icon
   */

  protected ImageIcon createImageIcon(String path)
  {
    java.net.URL imgURL = getClass().getResource(path);
    ImageIcon    image  = null;

    if (imgURL != null)
    {
      image = new ImageIcon(imgURL);
    }
    else
    {
       System.err.println("Couldn't find file: " + path);
    }

    return (image);
  }

  /* ===================================================================== *
   * -------------------------- CLOSE DIALOG ----------------------------- *
   * ===================================================================== */

  /** Method close()
   * <p> This method will close all dialogs that are opened by this class.
   */

  public void close()
  {
    // Input Variables

    notifier   = null; // Top Observables Object
    stCMN      = null; // Common Data Structure

//    if (pnlWell != null)
//      pnlWell.close();
//    pnlWell  = null;
  }

  /* ===================================================================== *
   * -------------------------- GET METHODS ------------------------------ *
   * ===================================================================== */


  /* ===================================================================== *
   * ---------------------------- ACTIONS -------------------------------- *
   * ===================================================================== */

  /** METHOD actionPerformed()
   *  <p> This Method will handle all the actions within the Frame.
   *  @param event - Action Event
   */

  public void actionPerformed(ActionEvent event)
  {
  }
}

/*
 *  @version 1.1 05/12/2015
 *  @author  John Victorine
 */
