/*
 * @quakeSensorDataPanel.java Version 1.1 05/21/2015
 *
 * Copyright (c) 2015 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package quake.gui;

import java.io.*;
import java.net.*;
import java.lang.*;
import java.util.*;
import java.util.Observer;
import java.util.Observable;
import java.awt.*;
import java.awt.event.*;
import javax.swing.border.*;

import javax.swing.*;
import javax.swing.event.*;

import cmn.cmnStruct;                    // Common Data Structure
import quake.gui.quakeSensorDataTable;   // Sensor Data Table
import quake.quakeSensorsDataStruct;     // Computed Quake Location Data Structure
import quake.quakeSensorsDataListStruct; // Compute Quake Location List Structure
import quake.quakeSensorsStruct;         // Sensor Data Structure

import gui.guiSearchDirectoryFrame;      // File Search Dialog

/** Class quakeSensorDataPanel
 *  <p> This Class will create the seismic event location data Panel
 *
 *  @version 1.1 05/21/2015
 *  @author  John R. Victorine
 */

public class quakeSensorDataPanel extends JPanel implements ActionListener, Observer
{
  // Input Variables

  private cmnStruct  stCMN     = null; // Common Data Structure
  private Observable pNotifier = null; // Observables Object

  // Global Variables

  private Observable notifier = null; // Observables Object

  private quakeSensorDataTable       pTable  = null; // Sensor data Table
  private quakeSensorsDataListStruct stList  = null; // Sensor list Structure

 private quakeSensorsStruct         stSensor = null; // Seismic Sensor Data

 private guiSearchDirectoryFrame    pSearch  = null; // Search for Directory to Save PNG

  public static final int _ADD    = 0;
  public static final int _MODIFY = 1;

  private int iAction = _ADD;

  private static final int _SQL = 0;
  private static final int _CSV = 1;

  private int    iSave = -1;

  private String sPath = new String("");
  private String sName = new String("");

  // Global Panel Widgets

  private JButton     btnCSV       = new JButton(); // Save as CSV File
  private JButton     btnSQL       = new JButton(); // Save as SQL File

  private JButton     btnModify    = new JButton(); // Modify
  private JButton     btnDelete    = new JButton(); // Delete
  private JButton     btnDeleteAll = new JButton(); // Delete All

  /** Constructor quakeSensorDataPanel()
   *  <p> This is the Constructor for this class.
   *  @param stCMN     = Comman Data Structure
   *  @param pNotifier = Observable
   */

  public quakeSensorDataPanel( cmnStruct stCMN, Observable pNotifier )
  {
    try
    {
      this.stCMN    = stCMN;
      this.pNotifier = pNotifier;

      jbInit();
      setButtons();
    }
    catch(Exception e)
    {
      e.printStackTrace();
    }
  }

  /** METHOD jbInit()
   *  <p> This Method will create the Frame for this class.
   *  @throws Exception
   */

  private void jbInit() throws Exception
  {
	JPanel      pnlTop    = new JPanel();
	JPanel      pnlCenter = new JPanel();
	JPanel      pnlBase   = new JPanel();

	JPanel      pBaseL    = new JPanel();
	JPanel      pBaseL2   = new JPanel();
	JPanel      pBaseR	  = new JPanel();

    JButton     btn       = new JButton();

    JScrollPane scroll    = new JScrollPane();

    // Create an Observable

    notifier = new quakeSensorDataPanelObservable();

    // Add this Dialog to the notifier as observer

    notifier.addObserver(this);

    // Build Panel

    this.setLayout(new BorderLayout());

    // Center Panel

    pnlCenter.setLayout(new BorderLayout());

    pTable = new quakeSensorDataTable();
    scroll = pTable.getScrollPane();

    // Base Buttons Panel

    pnlBase.setLayout(new BorderLayout());

    // -- Left Buttons Panel

    pBaseL.setLayout(new BorderLayout());
    pBaseL2.setLayout(new GridLayout());

    btnCSV.setFont(new java.awt.Font("Dialog", 1, 11));
    btnCSV.setBorderPainted(true);
    btnCSV.setText("Save as CSV File");
    btnCSV.addActionListener(this);

    btnSQL.setFont(new java.awt.Font("Dialog", 1, 11));
    btnSQL.setBorderPainted(true);
    btnSQL.setText("Save as SQL File");
    btnSQL.addActionListener(this);

    // -- Right Buttons Panel

    pBaseR.setLayout(new GridLayout());

    btnModify.setFont(new java.awt.Font("Dialog", 1, 11));
    btnModify.setBorderPainted(true);
    btnModify.setText("Modify Row");
    btnModify.addActionListener(this);

    btnDelete.setFont(new java.awt.Font("Dialog", 1, 11));
    btnDelete.setBorderPainted(true);
    btnDelete.setText("Delete Row");
    btnDelete.addActionListener(this);

    btnDeleteAll.setFont(new java.awt.Font("Dialog", 1, 11));
    btnDeleteAll.setBorderPainted(true);
    btnDeleteAll.setText("Delete All Rows");
    btnDeleteAll.addActionListener(this);

    // Attach Widgets

    this.add(pnlCenter,      BorderLayout.CENTER );
    pnlCenter.add( scroll,   BorderLayout.CENTER );

    this.add(pnlBase,        BorderLayout.SOUTH );
    pnlBase.add(pBaseL,      BorderLayout.WEST );
    pBaseL.add(pBaseL2,      BorderLayout.CENTER );
    pBaseL2.add(btnCSV,      null);
    pBaseL2.add(btnSQL,      null);
    pBaseL.add(btn,          BorderLayout.EAST );

    pnlBase.add(pBaseR,      BorderLayout.CENTER );
    pBaseR.add(btnModify,    null);
    pBaseR.add(btnDelete,    null);
    pBaseR.add(btnDeleteAll, null);
  }

  /* ===================================================================== *
   * -------------------------- CLOSE DIALOG ----------------------------- *
   * ===================================================================== */

  /** Method close()
   * <p> This method will close all dialogs that are opened by this class.
   */

  public void close()
  {
    // Input Variables

    pNotifier   = null; // Top Observables Object
    stCMN      = null; // Common Data Structure

    // Global Variables

    notifier = null; // Observables Object

    if (pTable != null)
      pTable.close();
    pTable = null; // Sensor data Table

    if (stList != null)
      stList.delete();
    stList = null;

    if (pSearch != null)
      pSearch.close();
    pSearch = null;

    sPath = null;
    sName = null;

    stSensor = null; // Seismic Sensor Data

    // Global Panel Widgets

    btnCSV       = null; // Save as CSV File
    btnSQL       = null; // Save as SQL File

    btnModify    = null; // Modify
    btnDelete    = null; // Delete
    btnDeleteAll = null; // Delete All
  }

  /* ===================================================================== *
   * -------------------------- GET METHODS ------------------------------ *
   * ===================================================================== */

  /** Method getData()
   * <p> This method will add the quake location to the list
   * @return st = the quake location data structure
   */

  public quakeSensorsDataStruct getData()
  {
	quakeSensorsDataStruct st = null;

    iAction = _MODIFY;

    if (pTable != null)
    {
	  if (pTable.getTotalRows() > 0)
	  {
		st = pTable.getRowData();
	  }
	}

    return (st);
  }

  /* ===================================================================== *
   * -------------------------- SET METHODS ------------------------------ *
   * ===================================================================== */

  /** Method setButtons()
   *  <p> This method will enable and disable buttons depending on data
   */

  private void setButtons()
  {
    btnCSV.setEnabled(false);        // Save as CSV File
    btnSQL.setEnabled(false);        // Save as SQL File

    btnModify.setEnabled(false);     // Modify
    btnDelete.setEnabled(false);     // Delete
    btnDeleteAll.setEnabled(false);  // Delete All

    if ((pTable.getTotalRows() > 0) && (iAction == _ADD))
    {
      btnCSV.setEnabled(true);       // Save as CSV File
      btnSQL.setEnabled(true);       // Save as SQL File

      btnModify.setEnabled(true);    // Modify
      btnDelete.setEnabled(true);    // Delete
      btnDeleteAll.setEnabled(true); // Delete All
	}
  }

  /** Method setSensors()
   *  <p> This method will set the sensor data
   *  @param st = The Sensor data structure
   */

  public void setSensors( quakeSensorsStruct st ) { this.stSensor = st; }

  /** Method setData()
   * <p> This method will add the quake location to the list
   * @param st = the quake location data structure
   */

  public void setData(quakeSensorsDataStruct st)
  {
	String sKEY = "0";

	if (iAction == _ADD)
	{
	  stList = quake.quakeSensorsDataUtility.add( st, stList );
	}
	else if (iAction == _MODIFY)
	{
	  sKEY   = new String(st.sKEY);
	  stList = quake.quakeSensorsDataUtility.modify( sKEY, st, stList );
	}

    if (pTable != null)
    {
      pTable.setData( stList );
    }

    iAction = _ADD;
    setButtons();
  }

  /** Method clear()
   *  <p> This method will clear the modifier indicator
   */

  public void clear()
  {
    iAction = _ADD;
    setButtons();
  }

  /** Method modify()
   * <p> This method will modify the quake location to the list
   */

  public void modify()
  {
    if (pNotifier != null)
      pNotifier.notifyObservers( new String("Modify Quake Data") );
  }

  /** Method delete()
   * <p> This method will delete row from table
   */

  public void delete()
  {
	quakeSensorsDataStruct st = null;
	String sKEY = "0";

    if (pTable != null)
    {
	  if (pTable.getTotalRows() > 0)
	  {
		st     = pTable.getRowData();
  	    sKEY   = new String(st.sKEY);
		stList = quake.quakeSensorsDataUtility.remove( sKEY, stList );

        pTable.setData( stList );
	  }
	}
  }

  /** Method deleteAll()
   * <p> This method will delete row from table
   */

  public void deleteAll()
  {
	if (stList != null)
	  stList.delete();
	stList = null;

    if (pTable != null)
    {
      pTable.setData( stList );
    }
  }

  /* ===================================================================== *
   * --------------------------- WRITE DATA ------------------------------ *
   * ===================================================================== */

  /** Method createSQL()
   *  <p> This method will create a ASCII SQL File of Production Data
   */

  private void createSQL()
  {
	String sSQL = "";

	sSQL = quake.io.WriteSQLFile.buildSQL( stList, stSensor );
	quake.io.WriteSQLFile.write( sPath, sName+".sql", sSQL );
  }

  /** Method createCSV()
   *  <p> This method will create a ASCII CSV File of Production Data
   */

  private void createCSV()
  {
	String sCSV = "";

	sCSV = quake.io.WriteCSVFile.buildCSV( stList );
	quake.io.WriteCSVFile.write( sPath, sName+".csv", sCSV );
  }

  /* ===================================================================== *
   * ---------------------------- ACTIONS -------------------------------- *
   * ===================================================================== */

  /** METHOD actionPerformed()
   *  <p> This Method will handle all the actions within the Frame.
   *  @param event - Action Event
   */

  public void actionPerformed(ActionEvent event)
  {
    if (event.getSource() == btnCSV) // Save as CSV File
    {
	  iSave = _CSV;
	  if (sPath.length() == 0)
        sPath = kgs.io.kgsIOUser.getHomeDir();

      if (sName.length() == 0)
        sName = new String( "events" );

      pSearch = new guiSearchDirectoryFrame(notifier, sPath, sName, ".csv");
	}

    if (event.getSource() == btnSQL) // Create a PDF of Profile Plot
    {
	  iSave = _SQL;
	  if (sPath.length() == 0)
        sPath = kgs.io.kgsIOUser.getHomeDir();

      if (sName.length() == 0)
        sName = new String( "events" );

      pSearch = new guiSearchDirectoryFrame(notifier, sPath, sName, ".sql");
    }

    if (event.getSource() == btnModify)    { modify(); }    // Modify
    if (event.getSource() == btnDelete)    { delete(); }    // Delete
    if (event.getSource() == btnDeleteAll) { deleteAll(); } // Delete All

    setButtons();
  }

  /** Method update()
   * <p> This method will handle the Observable Actions
   * @param obj = The Observable making the call
   * @param arg = The returned object
   */

  public void update(Observable obj, Object arg)
  {
    String sArg      = new String((String) arg);

    if (sArg.equals("Directory Path Selected"))
    {
      if (pSearch != null)
      {
        sPath = new String( pSearch.getPath() );
        sName = new String( pSearch.getFile() );
      }

      switch (iSave)
      {
        case _SQL:
          createSQL();
          break;
        case _CSV:
          createCSV();
          break;
	  }

      iSave = -1;

      if (pSearch != null)
        pSearch.close();
      pSearch = null;
    }
  }
}

/** CLASS quakeSensorDataPanelObservable()
  *  <p> This Class will handle Observable Actions when action occurs on a
  *      frame or panel
  */

class quakeSensorDataPanelObservable extends Observable
{
  public void notifyObservers(Object b)
  {
    setChanged();
    super.notifyObservers(b);
  }
}

/*
 *  @version 1.1 05/21/2015
 *  @author  John Victorine
 */
