/*
 * @WriteCSVFile.java Version 1.1 05/21/2015
 *
 * Copyright (c) 2015 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package quake.io;

import java.io.*;
import java.awt.*;

import util.utilFileIO;
import cmn.cmnString;

import quake.quakeSensorsDataListStruct; // Compute Quake Location List Structure

/** Class WriteCSVFile
 *  <p> This class will parse the List Structure and
 *      write the information to the CSV File.
 *
 *  @version 1.1 05/21/2015
 *  @author  John R. Victorine
 */

public class WriteCSVFile
{
  public static final int    _TOP       = event.eventStandardTools._TOP;
  public static final int    _BASE      = event.eventStandardTools._BASE;
  public static final int    _NAME      = event.eventStandardTools._NAME;
  public static final String CURVES[][] = event.eventStandardTools.CURVES;

  /** Method write()
   *  <p> This is the method will write the CSV String to the user's PC
   *  @param sPath     = The Directory Path for the CSV File
   *  @param sFilename = The Filename of the CSV File.
   *  @param sCSV      = The Headers CSV String
   */

  public static void write(String sPath, String sFilename, String sCSV)
  {
    utilFileIO pIO  = new utilFileIO();

    // Open the File for write

    pIO.Open( util.utilFileIO.DIRECTORY,
              util.utilFileIO.WRITE,
              sPath,
              sFilename );

    // Write the CSV Block to the File

    pIO.Write( sCSV );

    // Close the File

    pIO.Close();
  }

  /** Method buildCSV()
   *  <p> This method will build the CSV File
   *  @param  st   = quake location list data structure
   *  @return sCSV = ASCII CSV String
   */

  public static String buildCSV( quakeSensorsDataListStruct st )
  {
	String sCSV     = "";

    sCSV = new String("Date Time, Channel, Magnitude, Latitude, Longitude, Depth, " +
             "#1, PS Diff1, Vp1, Vs1, R1, " +
             "#2, PS Diff2, Vp2, Vs2, R2, " +
             "#3, PS Diff3, Vp3, Vs3, R3, \n");
    sCSV = new String(sCSV + "  , , , degrees, degrees, feet, " +
              " , msec, m/sec, m/sec, m, " +
              " , msec, m/sec, m/sec, m, " +
              " , msec, m/sec, m/sec, m, \n");

    if (st != null)
    {
	  for (int i=0; i<st.iCount; i++)
	  {
		sCSV = new String(sCSV + st.stItem[i].sTime      + ", " +
		                         st.stItem[i].sChannel   + ", " +
		                         st.stItem[i].dMagnitude + ", " +
		                         st.stItem[i].dLatitude  + ", " +
		                         st.stItem[i].dLongitude + ", " +
		                         st.stItem[i].depth      + ", ");

		for (int j=0; j<3; j++)
		{
		  sCSV = new String(sCSV + st.stItem[i].data[j][0] + ", " +
		                           st.stItem[i].data[j][1] + ", " +
		                           st.stItem[i].data[j][2] + ", " +
		                           st.stItem[i].data[j][3] + ", " +
		                           st.stItem[i].data[j][4] + ", ");
		}

        sCSV = new String(sCSV + "\n");
	  }
	}

	return (sCSV);
  }
}

/*
 *  @version 1.1 05/21/2015
 *  @author  John Victorine
 */
